/*
  tuxpaint.c

  Tux Paint - A simple drawing program for children.

  Copyright (c) 2002 by Bill Kendrick
  bill@newbreedsoftware.com
  http://www.newbreedsoftware.com/tuxpaint/

  June 14, 2002 - August 23, 2002
*/


#define VER_VERSION     "0.0.1pre"
#define VER_DATE        "2002.08.23"


/* #define DEBUG */
#define HIDE_UNIMPLEMENTED
/* #define LOW_QUALITY_THUMBNAILS */
/* #define LOW_QUALITY_COLOR_SELECTOR */
/* #define LOW_QUALITY_STAMP_OUTLINE */

#define MAX_FILES 128  /* Max. # of files in a dir. to worry about... */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <time.h>

/* Hopefully won't break Win32 ;) */
#include <locale.h>

#ifdef WIN32
/* The following are required by libintl.h, so must be defined first: */
#define LC_MESSAGES       1729
#define HAVE_LC_MESSAGES  1
#define ENABLE_NLS        1
#define HAVE_LOCALE_H     1
#define HAVE_GETTEXT      1
#define HAVE_DCGETTEXT    1
#endif

#if    defined(sun) && defined(__svr4__)
  /* Solaris needs locale.h */
#endif

#include <libintl.h>
#define gettext_noop(String) String

#ifndef M_PI
#define M_PI 3.14159265
#endif

#include <sys/types.h>
#include <sys/stat.h>


#ifndef WIN32
#include <unistd.h>
#include <dirent.h>
#else
#include "win32_dirent.h"
#include <io.h>
#define DATA_PREFIX "data/"
#define DOC_PREFIX "docs/"
#define LOCALEDIR "locale"
#define snprintf _snprintf
#define S_ISDIR(i) ((i&_S_IFDIR)!=0)
#endif

#include <errno.h>
#include <sys/stat.h>

#include "SDL.h"
#include "SDL_image.h"
#include "SDL_ttf.h"
#ifndef NOSOUND
#include "SDL_mixer.h"
#endif

#ifndef SAVE_AS_BMP
#include <png.h>
#define FNAME_EXTENSION ".png"
#else
#define FNAME_EXTENSION ".bmp"
#endif

#include "tools.h"
#include "titles.h"
#include "colors.h"
#include "shapes.h"
#include "magic.h"
#include "sounds.h"
#include "tip_tux.h"


#ifdef WIN32
/*
      The SDL stderr redirection trick doesn't seem to work for perror().
      This does pretty much the same thing.
*/
void win32_perror(const char *str)
{
    if ( str && *str )
      fprintf(stderr,"%s : ",str);
    fprintf(stderr,
            "%s [%d]\n",
            (errno<_sys_nerr)?_sys_errlist[errno]:"unknown",errno );
}
#define perror         win32_perror
#endif


#ifndef WIN32
#define min(a,b) ((a < b) ? a : b)
#define max(a,b) ((a > b) ? a : b)
#endif

#define clamp(lo,value,hi)    (min(max(value,lo),hi))


#ifdef WIN32
#define RENDER_TEXT TTF_RenderUTF8_Blended
#else
#define RENDER_TEXT TTF_RenderText_Blended
#endif


/* Possible languages: */

enum {
  LANG_DE,  /* German */
  LANG_EN,  /* English */
  LANG_ES,  /* Spanish */
  LANG_FI,  /* Finnish */
  LANG_FR,  /* French */
  LANG_IT,  /* Italian */
  LANG_NL,  /* Dutch */
  LANG_NN,  /* Norwegian */
  LANG_TR,  /* Turkish */
  NUM_LANGS
};

const char * lang_prefixes[NUM_LANGS] = {
  "de",
  "en",
  "es",
  "fi",
  "fr",
  "it",
  "nl",
  "nn",
  "tr"
};


typedef struct info_type {
  int colorable;
  int tintable;
} info_type;


/* Globals: */

int use_sound, fullscreen, disable_quit, simple_shapes, language;
int prog_bar_ctr;
SDL_Surface * screen;

SDL_Surface * canvas;

#define NUM_UNDO_BUFS 10
SDL_Surface * undo_bufs[NUM_UNDO_BUFS];
int cur_undo, oldest_undo, newest_undo;

SDL_Surface * img_title, * img_loading, * img_progress;
SDL_Surface * img_btn_up, * img_btn_down, * img_btn_off;
SDL_Surface * img_yes, * img_no;
SDL_Surface * img_open, * img_erase, * img_back;
SDL_Surface * img_cursor_up, * img_cursor_down;
SDL_Surface * img_scroll_up, * img_scroll_down;
SDL_Surface * img_scroll_up_off, * img_scroll_down_off;
SDL_Surface * img_paintcan;

SDL_Surface * img_sparkles;

SDL_Surface * img_titles[NUM_TITLES];
SDL_Surface * img_tools[NUM_TOOLS], * img_tool_names[NUM_TOOLS];

#define MAX_STAMPS 256
#define MAX_BRUSHES 64
int num_brushes, num_stamps;
SDL_Surface * img_brushes[MAX_BRUSHES];
SDL_Surface * img_stamps[MAX_STAMPS];
char * txt_stamps[MAX_STAMPS];
info_type * inf_stamps[MAX_STAMPS];
#ifndef NOSOUND
Mix_Chunk * snd_stamps[MAX_STAMPS];
#endif
SDL_Surface * img_stamp_thumbs[MAX_STAMPS];

SDL_Surface * img_shapes[NUM_SHAPES], * img_shape_names[NUM_SHAPES];
SDL_Surface * img_magics[NUM_MAGICS], * img_magic_names[NUM_MAGICS];

SDL_Surface * img_tux[NUM_TIP_TUX];

#ifndef LOW_QUALITY_COLOR_SELECTOR
SDL_Surface * img_color_btns[NUM_COLORS];
#endif

SDL_Surface * img_cur_brush;
int brush_counter, rainbow_color;

TTF_Font * font, * small_font;

#ifndef NOSOUND
Mix_Chunk * sounds[NUM_SOUNDS];
#endif


int cur_tool, cur_color, cur_brush, cur_stamp, cur_shape, cur_magic;
int been_saved;
char file_id[32];
int brush_scroll, stamp_scroll;
int eraser_sound;

int tool_avail[NUM_TOOLS], tool_avail_bak[NUM_TOOLS];

typedef struct edge_type {
  int y_upper;
  float x_intersect, dx_per_scan;
  struct edge_type * next;
} edge;


typedef struct point_type {
  int x, y;
} point_type;

typedef struct fpoint_type {
  float x, y;
} fpoint_type;

typedef enum { Left, Right, Bottom, Top } an_edge;
#define NUM_EDGES 4


/* Local function prototypes: */

void mainloop(void);
void brush_draw(int x1, int y1, int x2, int y2, int update);
void blit_brush(int x, int y);
void magic_draw(int x1, int y1, int x2, int y2);
void blit_magic(int x, int y, int x2, int y2);
void stamp_draw(int x, int y);
void rec_undo_buffer(void);
void update_canvas(int x1, int y1, int x2, int y2);
void setup(int argc, char * argv[]);
void seticon(void);
SDL_Surface * loadimage(char * fname);
void draw_toolbar(void);
void draw_magic(void);
void draw_colors(int enabled);
void draw_brushes(void);
void draw_stamps(void);
void draw_shapes(void);
void draw_none(void);
#ifndef NOSOUND
void loadarbitrary(SDL_Surface * surfs[], char * descs[], info_type * infs[],
		   Mix_Chunk * sounds[], int * count, int starting, int max,
		   char * dir);
#else
void loadarbitrary(SDL_Surface * surfs[], char * descs[], info_type * infs[],
		   int * count, int starting, int max,
		   char * dir);
#endif
SDL_Surface * thumbnail(SDL_Surface * src, int max_x, int max_y,
			int keep_aspect);
Uint32 getpixel(SDL_Surface * surface, int x, int y);
void putpixel(SDL_Surface * surface, int x, int y, Uint32 pixel);
void debug(char * str);
void do_undo(void);
void do_redo(void);
void render_brush(void);
void playsound(int chan, int s, int override);
void line_xor(int x1, int y1, int x2, int y2);
void clipped_putpixel(SDL_Surface * dest, int x, int y, Uint32 c);
void rect_xor(int x1, int y1, int x2, int y2);
void stamp_xor(int x1, int y1);
void do_eraser(int x, int y);
void disable_avail_tools(void);
void enable_avail_tools(void);
void reset_avail_tools(void);
void update_screen(int x1, int y1, int x2, int y2);
Uint8 alpha(Uint8 c1, Uint8 c2, Uint8 a);
int compare_strings(char * * s1, char * * s2);
void draw_tux_text(int which_tux, char * str);
void wordwrap_text(TTF_Font * font, char * str, SDL_Color color,
		   int left, int top, int right);
char * loaddesc(char * fname);
info_type * loadinfo(char * fname);
#ifndef NOSOUND
Mix_Chunk * loadsound(char * fname);
#endif
void do_wait(void);
void load_current(void);
void save_current(void);
char * get_fname(char * name);
int do_prompt(char * text, char * btn_yes, char * btn_no);
void cleanup(void);
void free_surface(SDL_Surface **surface_array);
void free_surface_array(SDL_Surface *surface_array[], int count);
void do_shape(int cx, int cy, int ox, int oy, int rotn, int use_brush);
int rotation(int ctr_x, int ctr_y, int ox, int oy);
void do_save(void);
void get_new_file_id(void);
int do_quit(void);
void do_open(void);
void scan_fill(int cnt, point_type * pts);
int clip_polygon(int n, fpoint_type * pin, fpoint_type * pout);
void wait_for_sfx(void);
int current_language(void);
int stamp_colorable(int stamp);
int stamp_tintable(int stamp);
void rgbtohsv(Uint8 r8, Uint8 g8, Uint8 b8, float *h, float *s, float *v);
void hsvtorgb(float h, float s, float v, Uint8 *r8, Uint8 *g8, Uint8 *b8);
void show_progress_bar(void);


/* --- MAIN --- */

int main(int argc, char * argv[])
{
  /* Set up locale support */

  setlocale(LC_ALL, "");


  /* Set up! */

  setup(argc, argv);


  SDL_BlitSurface(img_title, NULL, screen, NULL);
  SDL_Flip(screen);
  playsound(0, SND_HARP, 1);
  do_wait();
  SDL_FreeSurface(img_title);


  /* Set defaults! */

  cur_undo = 0;
  oldest_undo = 0;
  newest_undo = 0;

  cur_tool = TOOL_BRUSH;
  cur_color = COLOR_BLACK;
  cur_brush = 0;
  cur_stamp = 0;
  cur_shape = SHAPE_SQUARE;
  cur_magic = 0;

  eraser_sound = 0;

  img_cur_brush = NULL;
  render_brush();

  brush_scroll = 0;
  stamp_scroll = 0;


  reset_avail_tools();


  /* Load current image (if any): */

  load_current();
  been_saved = 1;
  tool_avail[TOOL_SAVE] = 0;


  /* Draw the screen! */

  SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 255, 255, 255));

  draw_toolbar();
  draw_colors(1);
  draw_brushes();
  update_canvas(0, 0, 640 - 96, (48 * 7) + 40);

  SDL_Flip(screen);


  /* Main loop! */

  mainloop();


  /* Close and quit! */

  save_current();

  wait_for_sfx();


  cleanup();

  return 0;
}


/* FIXME: Move elsewhere!!! */

#define PROMPT_QUIT_TXT gettext("Do you really want to quit?")
#define PROMPT_QUIT_YES gettext("Yes")
#define PROMPT_QUIT_NO  gettext("No")

#define PROMPT_QUIT_SAVE_TXT gettext("If you quit, you'll lose your picture! Save it?")
#define PROMPT_QUIT_SAVE_YES gettext("Yes")
#define PROMPT_QUIT_SAVE_NO gettext("No")

#define PROMPT_OPEN_SAVE_TXT gettext("Save your picture first?")
#define PROMPT_OPEN_SAVE_YES gettext("Yes")
#define PROMPT_OPEN_SAVE_NO gettext("No")

#define PROMPT_OPEN_UNOPENABLE_TXT gettext("Can't open that picture!")
#define PROMPT_OPEN_UNOPENABLE_YES gettext("Okay")

#define PROMPT_NEW_TXT gettext("Starting a new picture will erase the current one!")
#define PROMPT_NEW_YES gettext("That's Ok")
#define PROMPT_NEW_NO gettext("Never Mind!")

#define PROMPT_OPEN_NOFILES_TXT gettext("There are no saved files!")
#define PROMPT_OPEN_NOFILES_YES gettext("Okay")

#define PROMPT_ERASE_TXT gettext("Erase this picture?")
#define PROMPT_ERASE_YES gettext("Yes")
#define PROMPT_ERASE_NO gettext("No")


enum {
  SHAPE_TOOL_MODE_STRETCH,
  SHAPE_TOOL_MODE_ROTATE,
  SHAPE_TOOL_MODE_DONE
};


/* --- MAIN LOOP! --- */

void mainloop(void)
{
  int done, off_y, which, button_down, old_x, old_y, new_x, new_y,
    line_start_x, line_start_y, w, h, shape_tool_mode,
    shape_ctr_x, shape_ctr_y, shape_outer_x, shape_outer_y;
  int num_things, thing_scroll, cur_thing, old_thing, do_draw, old_tool,
    tmp_int;
  SDL_Event event;
  SDLKey key;
  SDLMod mod;


  num_things = 0;
  thing_scroll = 0;
  cur_thing = 0;
  old_thing = 0;
  do_draw = 0;
  old_x = 0;
  old_y = 0;
  new_x = 0;
  new_y = 0;
  line_start_x = 0;
  line_start_y = 0;
  shape_ctr_x = 0;
  shape_ctr_y = 0;
  shape_outer_x = 0;
  shape_outer_y =0;
  shape_tool_mode = SHAPE_TOOL_MODE_DONE;
  button_down = 0;


  done = 0;

  do
  {
    SDL_WaitEvent(&event);

    if (event.type == SDL_QUIT)
    {
      done = do_quit();
    }
    else if (event.type == SDL_KEYDOWN)
    {
      key = event.key.keysym.sym;
      mod = event.key.keysym.mod;

      if (key == SDLK_ESCAPE)
      {
        done = do_quit();
      }
      else if (key == SDLK_z &&
	       (mod == KMOD_CTRL ||
		mod == KMOD_LCTRL ||
		mod == KMOD_RCTRL))
      {
	/* Ctrl-Z - Undo */

	if (tool_avail[TOOL_UNDO])
	{
	  if (cur_undo == newest_undo)
	  {
	    rec_undo_buffer();
	    do_undo();
          }
          do_undo();
          SDL_UpdateRect(screen, 0, 0, 96, (48 * 7) + 40);
          shape_tool_mode = SHAPE_TOOL_MODE_DONE;
	}
      }
      else if (key == SDLK_r &&
	       (mod == KMOD_CTRL ||
		mod == KMOD_LCTRL ||
		mod == KMOD_RCTRL))
      {
	/* Ctrl-R - Redo */
	
	if (tool_avail[TOOL_REDO])
	{
          do_redo();
          SDL_UpdateRect(screen, 0, 0, 96, (48 * 7) + 40);
          shape_tool_mode = SHAPE_TOOL_MODE_DONE;
	}
      }
      else if (key == SDLK_o &&
	       (mod == KMOD_CTRL ||
		mod == KMOD_LCTRL ||
		mod == KMOD_RCTRL))
      {
	/* Ctrl-O - Open */
	
	disable_avail_tools();
	draw_toolbar();
	
	do_open();
	
	tmp_int = tool_avail[TOOL_NEW];
	enable_avail_tools();
        tool_avail[TOOL_NEW] = tmp_int;
	
	draw_toolbar();
        SDL_UpdateRect(screen, 0, 0, 96, (48 * 7) + 40);
      }
      else if (key == SDLK_s &&
	       (mod == KMOD_CTRL ||
		mod == KMOD_LCTRL ||
		mod == KMOD_RCTRL))
      {
	/* Ctrl-S - Save */
	
        do_save();

	been_saved = 1;
	tool_avail[TOOL_SAVE] = 0;
	
	SDL_UpdateRect(screen, 0, 0, 96, (48 * 7) + 40);
      }
    }
    else if (event.type == SDL_MOUSEBUTTONDOWN)
    {
      if (/* event.button.x >= 0 && */
          event.button.x < 96 &&
	  event.button.y >= 40 &&
	  event.button.y < (48 * 7) + 40)
      {
	/* A tool on the left has been pressed! */

	which = ((event.button.y - 40) / 48) * 2 +
	        (event.button.x / 48);

	if (which < NUM_TOOLS && tool_avail[which])
	{
	  old_tool = cur_tool;
          cur_tool = which;
	  draw_toolbar();
          SDL_UpdateRect(screen, 0, 0, 96, (48 * 7) + 40);

	  playsound(1, SND_CLICK, 0);

	  draw_tux_text(tool_tux[cur_tool], gettext(tool_tips[cur_tool]));
	
	
	  /* Draw items for this tool: */
	
	  if (cur_tool == TOOL_BRUSH)
	  {
	    draw_brushes();
	    draw_colors(1);
	  }
	  else if (cur_tool == TOOL_STAMP)
	  {
	    draw_stamps();
	    draw_colors(stamp_colorable(cur_stamp) ||
			stamp_tintable(cur_stamp));
	  }
	  else if (cur_tool == TOOL_LINES)
	  {
	    draw_brushes();
	    draw_colors(1);
	  }
	  else if (cur_tool == TOOL_SHAPES)
	  {
	    draw_shapes();
	    draw_colors(1);
	    shape_tool_mode = SHAPE_TOOL_MODE_DONE;
	  }
	  else if (cur_tool == TOOL_TEXT)
	  {
	    draw_none(); /* FIXME: Draw fonts */
	    draw_colors(1);
	  }
	  else if (cur_tool == TOOL_ERASER)
	  {
	    draw_none();
	    draw_colors(0);
	  }
	  else if (cur_tool == TOOL_UNDO)
	    {
	      if (cur_undo == newest_undo)
	      {
		rec_undo_buffer();
		do_undo();
	      }
	      do_undo();
	
	      been_saved = 0;
	      tool_avail[TOOL_SAVE] = 1;
	
	      cur_tool = old_tool;
	      draw_toolbar();
              SDL_UpdateRect(screen, 0, 0, 96, (48 * 7) + 40);
              shape_tool_mode = SHAPE_TOOL_MODE_DONE;
	    }
	  else if (cur_tool == TOOL_REDO)
	    {
	      do_redo();
	
	      been_saved = 0;
	      tool_avail[TOOL_SAVE] = 1;

	      cur_tool = old_tool;
	      draw_toolbar();
              SDL_UpdateRect(screen, 0, 0, 96, (48 * 7) + 40);
              shape_tool_mode = SHAPE_TOOL_MODE_DONE;
	    }
	  else if (cur_tool == TOOL_OPEN)
	  {
	    disable_avail_tools();
	    draw_toolbar();
	    
	    do_open();
	   
	    tmp_int = tool_avail[TOOL_NEW];
	    enable_avail_tools();
            tool_avail[TOOL_NEW] = tmp_int;
	    
	    cur_tool = old_tool;
	    draw_toolbar();
            SDL_UpdateRect(screen, 0, 0, 96, (48 * 7) + 40);
	  }
	  else if (cur_tool == TOOL_SAVE)
	  {
            do_save();

	    been_saved = 1;
	    tool_avail[TOOL_SAVE] = 0;
	    cur_tool = old_tool;
	    draw_toolbar();
            SDL_UpdateRect(screen, 0, 0, 96, (48 * 7) + 40);
	  }
	  else if (cur_tool == TOOL_NEW)
	    {
              if (do_prompt(PROMPT_NEW_TXT,
		            PROMPT_NEW_YES,
		            PROMPT_NEW_NO))
	      {
	        SDL_FillRect(canvas, NULL,
			     SDL_MapRGB(canvas->format, 255, 255, 255));
	        update_canvas(0, 0, 640 - 96, (48 * 7) + 40);
	
	        cur_undo = 0;
	        oldest_undo = 0;
	        newest_undo = 0;
		shape_tool_mode = SHAPE_TOOL_MODE_DONE;
	
		been_saved = 1;
	        reset_avail_tools();

		file_id[0] = '\0';

		playsound(1, SND_HARP, 1);
	      }
	      else
	      {
		draw_tux_text(tool_tux[TUX_DEFAULT], TIP_NEW_ABORT);
	      }
	
	      cur_tool = old_tool;
	      draw_toolbar();
              SDL_UpdateRect(screen, 0, 0, 96, (48 * 7) + 40);
	    }
	  else if (cur_tool == TOOL_PRINT)
	    draw_none(); /* FIXME: Print! */
	  else if (cur_tool == TOOL_MAGIC)
	  {
	    rainbow_color = 0;
	    draw_magic();
	    draw_colors(0);
	  }
	  else if (cur_tool == TOOL_QUIT)
	  {
	    done = do_quit();
	    cur_tool = old_tool;
	    draw_toolbar();
	    SDL_UpdateRect(screen, 0, 0, 96, (48 * 7) + 40);
	  }

	  SDL_UpdateRect(screen, 640 - 96, 0, 96, (48 * 7) + 40);
	  SDL_UpdateRect(screen, 0, (48 * 7) + 40, 640, 48);
	}
      }
      else if (event.button.x >= 640 - 96 &&
          event.button.x < 640 &&
	  event.button.y >= 40 &&
	  event.button.y < (48 * 7) + 40)
      {
	/* Options on the right have been pressed! */
	
	if (cur_tool == TOOL_BRUSH || cur_tool == TOOL_STAMP ||
	    cur_tool == TOOL_SHAPES || cur_tool == TOOL_LINES ||
	    cur_tool == TOOL_MAGIC)
	  {
	    /* Note set of things we're dealing with (stamps, brushes, etc.) */
	
	    if (cur_tool == TOOL_BRUSH || cur_tool == TOOL_LINES)
	      {
		num_things = num_brushes;
		thing_scroll = brush_scroll;
	      }
	    else if (cur_tool == TOOL_STAMP)
	      {
		num_things = num_stamps;
		thing_scroll = stamp_scroll;
	      }
	    else if (cur_tool == TOOL_SHAPES)
	      {
		num_things = NUM_SHAPES;
		thing_scroll = 0;
	      }
	    else if (cur_tool == TOOL_MAGIC)
	      {
		num_things = NUM_MAGICS;
		thing_scroll = 0;
	      }
	
	    do_draw = 0;
	
	
	    /* Deal with scrollbars: */
	
	    if (num_things > 14)
	      {
		if (event.button.y < 40 + 24)
		  {
		    if (thing_scroll > 0)
		      {
			thing_scroll = thing_scroll - 2;
			do_draw = 1;

			playsound(1, SND_SCROLL, 1);
		      }
		  }
		else if (event.button.y >= (48 * 6) + 40 + 24)
		  {
		    if (thing_scroll < num_things - 12)
		      {
			thing_scroll = thing_scroll + 2;
			do_draw = 1;

			playsound(1, SND_SCROLL, 1);
		      }
		  }
		
		off_y = 24;
	      }
	    else
	      {
		off_y = 0;
	      }
	
	
	    if (event.button.y > 40 + off_y &&
		event.button.y < (48 * 7) + 40 - off_y)
	      {
		which = ((event.button.y - 40 - off_y) / 48) * 2 +
		  ((event.button.x - (640 - 96)) / 48) +
		  thing_scroll;
		
		if (which < num_things)
		  {
#ifndef NOSOUND
		    if (cur_tool != TOOL_STAMP ||
			snd_stamps[which] == NULL)
		    { 
		      playsound(1, SND_BLEEP, 0);
		    }
#endif

		    old_thing = cur_thing;
		    cur_thing = which;
		    do_draw = 1;
		  }
	      }

	
	    /* Assign the change(s), if any, and redraw, if necessary: */

	    if (cur_tool == TOOL_BRUSH || cur_tool == TOOL_LINES)
	      {
		cur_brush = cur_thing;
		brush_scroll = thing_scroll;
		render_brush();
		
		if (do_draw)
		  draw_brushes();
	      }
	    else if (cur_tool == TOOL_STAMP)
	      {
#ifndef NOSOUND
		if (cur_stamp != cur_thing)
		{
		  /* Only play when picking a different stamp, not
		     simply scrolling */

		  if (snd_stamps[cur_thing] != NULL)
		    Mix_PlayChannel(2, snd_stamps[cur_thing], 0);
		}
#endif
		cur_stamp = cur_thing;
		stamp_scroll = thing_scroll;
		
		if (do_draw)
		  draw_stamps();
	
		if (txt_stamps[cur_stamp] != NULL)
		  draw_tux_text(TUX_GREAT, txt_stamps[cur_stamp]);
		else
		  draw_tux_text(TUX_GREAT, "");


		/* Enable or disable color selector: */

		if ((stamp_colorable(cur_stamp) || stamp_tintable(cur_stamp)) !=
		    (stamp_colorable(old_thing) || stamp_tintable(old_thing)))
		{
		  draw_colors(stamp_colorable(cur_stamp) ||
			      stamp_tintable(cur_stamp));
	          SDL_UpdateRect(screen, 0, (48 * 7) + 40, 640, 48);
		}
	      }
	    else if (cur_tool == TOOL_SHAPES)
	      {
		cur_shape = cur_thing;

		draw_tux_text(TUX_GREAT, gettext(shape_tips[cur_shape]));
		
		if (do_draw)
		  draw_shapes();
	      }
	    else if (cur_tool == TOOL_MAGIC)
	      {
		cur_magic = cur_thing;
		
		draw_tux_text(TUX_GREAT, gettext(magic_tips[cur_magic]));

		if (do_draw)
		  draw_magic();
	      }
	
	
	    /* Update the screen: */
	
	    if (do_draw)
	      SDL_UpdateRect(screen, 640 - 96, 0, 96, (48 * 7) + 40);
	  }
      }
      else if (event.button.x > 96 &&
	       event.button.x < 640 && /* FIXME: Need exact number here! */
	       event.button.y > (48 * 7) + 40 &&
	       event.button.y <= (48 * 7) + 48 + 48 &&
	       (cur_tool == TOOL_BRUSH || cur_tool == TOOL_LINES ||
		cur_tool == TOOL_SHAPES || cur_tool == TOOL_TEXT ||
		(cur_tool == TOOL_STAMP &&
		 (stamp_colorable(cur_stamp) ||
		  stamp_tintable(cur_stamp)))))
      {
	/* Color! */

	which = (event.button.x - 96) / ((640 - 96) / NUM_COLORS);

	if (which < NUM_COLORS)
	{
	  cur_color = which;
	  playsound(1, SND_BUBBLE, 1);
	  draw_colors(1);
	  SDL_UpdateRect(screen, 0, (48 * 7) + 40, 640, 48);
	  render_brush();
	  draw_tux_text(TUX_KISS, gettext(color_names[cur_color]));
	}
      }
      else if (event.button.x > 96 && event.button.x < 640 - 96 &&
	       /* event.button.y >= 0 && */ event.button.y < (48 * 7) + 40)
	{
	  /* Draw something! */
	
	  button_down = 1;

	  old_x = event.button.x - 96;
	  old_y = event.button.y;

	  if (been_saved)
	  {
	    been_saved = 0;
	    tool_avail[TOOL_SAVE] = 1;
	    draw_toolbar();
	    SDL_UpdateRect(screen, 0, 0, 96, 48 * 7 + 40);
	  }
	
	  if (cur_tool == TOOL_BRUSH)
	    {
	      /* Start painting! */
	
	      rec_undo_buffer();
	
	      brush_counter = 999; /* arbitrarily large, so we draw once now */
	
	      brush_draw(old_x, old_y, old_x, old_y, 1);
	      playsound(0, SND_PAINT1 + (img_cur_brush->w) / 12, 1);
	    }
	  else if (cur_tool == TOOL_STAMP)
	    {
	      /* Draw a stamp! */
	
	      rec_undo_buffer();

	      stamp_draw(old_x, old_y);
#ifdef LOW_QUALITY_STAMP_OUTLINE
	      rect_xor(old_x - (img_stamps[cur_stamp]->w / 2),
		       old_y - (img_stamps[cur_stamp]->h / 2),
		       old_x + (img_stamps[cur_stamp]->w / 2),
		       old_y + (img_stamps[cur_stamp]->h / 2));
#else
	      stamp_xor(old_x - (img_stamps[cur_stamp]->w / 2),
			old_y - (img_stamps[cur_stamp]->h / 2));
#endif
	      playsound(1, SND_STAMP, 1);
	      draw_tux_text(TUX_KISS, TIP_STAMP_DRAW);
	    }
	  else if (cur_tool == TOOL_LINES)
	    {
	      /* Start a line! */
	
	      rec_undo_buffer();

	      line_start_x = old_x;
	      line_start_y = old_y;

	      brush_counter = 999; /* arbitrarily large, so we draw once now */

              brush_draw(old_x, old_y, old_x, old_y, 1);

	      playsound(1, SND_LINE_START, 1);
	      draw_tux_text(TUX_BORED, TIP_LINE_START);
	    }
	  else if (cur_tool == TOOL_SHAPES)
	    {
	      if (shape_tool_mode == SHAPE_TOOL_MODE_DONE)
	      {
	        /* Start drawing a shape! */
	
	        rec_undo_buffer();

	        shape_ctr_x = old_x;
	        shape_ctr_y = old_y;

	        shape_tool_mode = SHAPE_TOOL_MODE_STRETCH;

	        playsound(1, SND_LINE_START, 1);
	        draw_tux_text(TUX_BORED, TIP_SHAPE_START);
	      }
	      else if (shape_tool_mode == SHAPE_TOOL_MODE_ROTATE)
	      {
	        /* Draw the shape with the brush! */

	        brush_counter = 999; /* arbitrarily large... */

	        playsound(1, SND_LINE_END, 1);
	        do_shape(shape_ctr_x, shape_ctr_y,
		         shape_outer_x, shape_outer_y,
			 rotation(shape_ctr_x, shape_ctr_y,
			          event.button.x - 96, event.button.y),
		         1);

		shape_tool_mode = SHAPE_TOOL_MODE_DONE;
	        draw_tux_text(TUX_GREAT, gettext(tool_tips[TOOL_SHAPES]));
	      }
	    }
	  else if (cur_tool == TOOL_MAGIC)
	    {
	      /* Start doing magic! */
	
	      rec_undo_buffer();

	      brush_counter = 999; /* arbitrarily large, so we draw once now */

	      magic_draw(old_x, old_y, old_x, old_y);

	      if (cur_magic == MAGIC_FLIP ||
	          cur_magic == MAGIC_MIRROR)
	      {
		update_canvas(0, 0, canvas->w, canvas->h);
	      }
	    }
	  else if (cur_tool == TOOL_ERASER)
	    {
	      /* Erase! */

	      rec_undo_buffer();

	      do_eraser(old_x, old_y);
	    }
	  else if (cur_tool == TOOL_TEXT)
	    {
	      /* Text Tool! */
	
	      /* FIXME: Do something! */

	      debug("Text tool disabled!");
	    }

	
	  /* Make sure these commands are available now: */
	
	  if (tool_avail[TOOL_NEW] == 0)
	  {
	    tool_avail[TOOL_NEW] = 1;

	    draw_toolbar();
            SDL_UpdateRect(screen, 0, 0, 96, (48 * 7) + 40);
	  }
	}
    }
    else if (event.type == SDL_MOUSEBUTTONUP)
      {
	if (button_down)
	{
	  if (cur_tool == TOOL_LINES)
	    {
	      brush_counter = 999; /* arbitrarily large, so we draw once now */

              brush_draw(line_start_x, line_start_y,
			 event.button.x - 96, event.button.y, 1);
	      brush_draw(event.button.x - 96, event.button.y,
			 event.button.x - 96, event.button.y, 1);

	      playsound(1, SND_LINE_END, 1);
	      draw_tux_text(TUX_GREAT, gettext(tool_tips[TOOL_LINES]));
	    }
	  else if (cur_tool == TOOL_SHAPES)
	    {
	      if (shape_tool_mode == SHAPE_TOOL_MODE_STRETCH)
	      {
		/* Now we can rotate the shape... */

		shape_outer_x = event.button.x - 96;
		shape_outer_y = event.button.y;

		if (!simple_shapes)
		{
		  shape_tool_mode = SHAPE_TOOL_MODE_ROTATE;
	

		  /* Erase stretchy XOR: */
		
		  do_shape(shape_ctr_x, shape_ctr_y, old_x, old_y,
		         0, 0);

		  /* Make an initial rotation XOR to be erased: */

	          do_shape(shape_ctr_x, shape_ctr_y,
	                  shape_outer_x, shape_outer_y,
		          rotation(shape_ctr_x, shape_ctr_y,
			           shape_outer_x, shape_outer_y), 0);

	          playsound(1, SND_LINE_START, 1);
	          draw_tux_text(TUX_BORED, TIP_SHAPE_NEXT);

		
		  /* FIXME: Do something less intensive! */

		  SDL_Flip(screen);
		}
		else
		{
	          brush_counter = 999; /* arbitrarily large... */

	          playsound(1, SND_LINE_END, 1);
	          do_shape(shape_ctr_x, shape_ctr_y,
		           shape_outer_x, shape_outer_y,
			   0, 1);

		  shape_tool_mode = SHAPE_TOOL_MODE_DONE;
	          draw_tux_text(TUX_GREAT, gettext(tool_tips[TOOL_SHAPES]));
		}
	      }
	    }
	}

	button_down = 0;
      }
    else if (event.type == SDL_MOUSEMOTION)
      {
	new_x = event.button.x - 96;
	new_y = event.button.y;

	
	if (button_down)
	  {
	    if (cur_tool == TOOL_BRUSH)
	      {
		/* Pushing button and moving: Draw with the brush: */
		
		brush_draw(old_x, old_y, new_x, new_y, 1);

	        playsound(0, SND_PAINT1 + (img_cur_brush->w) / 12, 0);
	      }
	    else if (cur_tool == TOOL_LINES)
	      {
		/* Still pushing button, while moving:
		   Draw XOR where line will go: */
		
		line_xor(line_start_x, line_start_y, old_x, old_y);

		line_xor(line_start_x, line_start_y, new_x, new_y);

		update_screen(line_start_x + 96, line_start_y,
			      old_x + 96, old_y);
		update_screen(line_start_x + 96, line_start_y,
			      new_x + 96, new_y);
	      }
	    else if (cur_tool == TOOL_SHAPES)
	      {
		/* Still pushing button, while moving:
		   Draw XOR where shape will go: */

		if (shape_tool_mode == SHAPE_TOOL_MODE_STRETCH)
		{
	          do_shape(shape_ctr_x, shape_ctr_y, old_x, old_y,
		          0, 0);
		
	          do_shape(shape_ctr_x, shape_ctr_y,
		           new_x, new_y,
		           0, 0);

		  /* FIXME: Do something less intensive! */

		  SDL_Flip(screen);
		}
	      }
	    else if (cur_tool == TOOL_MAGIC)
	      {
		/* Pushing button and moving: Do the magic: */
		
		if (cur_magic != MAGIC_FLIP &&
		    cur_magic != MAGIC_MIRROR)
		{
		  magic_draw(old_x, old_y, new_x, new_y);
		}
	      }
	    else if (cur_tool == TOOL_ERASER)
	      {
		/* Still pushing, and moving - Erase! */
		
		do_eraser(new_x, new_y);
	      }
	  }


	if (cur_tool == TOOL_STAMP ||
	    (cur_tool == TOOL_ERASER && !button_down))
	  {
            /* Moving: Draw XOR where stamp/eraser will apply: */

	    if (cur_tool == TOOL_STAMP)
	    {
	      w = img_stamps[cur_stamp]->w;
	      h = img_stamps[cur_stamp]->h;
	    }
	    else if (cur_tool == TOOL_ERASER)
	    {
	      w = 96;
	      h = 96;
	    }

	    if (old_x >= 0 && old_x < 640 - 96 - 96 &&
	        old_y >= 0 && old_y < (48 * 7) + 40)
	    {
#ifndef LOW_QUALITY_STAMP_OUTLINE
	      if (cur_tool == TOOL_STAMP)
	        stamp_xor(old_x - w / 2, old_y - h / 2);
	      else
	        rect_xor(old_x - w / 2, old_y - h / 2,
		         old_x + w / 2, old_y + h / 2);
#else
	      rect_xor(old_x - w / 2, old_y - h / 2,
		       old_x + w / 2, old_y + h / 2);
#endif
	      update_screen(old_x - w / 2 + 96, old_y - h / 2,
		            old_x + w / 2 + 96, old_y + h / 2);
	    }

	    if (new_x >= 0 && new_x < 640 - 96 - 96 &&
	        new_y >= 0 && new_y < (48 * 7) + 40)
	    {
#ifndef LOW_QUALITY_STAMP_OUTLINE
	      if (cur_tool == TOOL_STAMP)
	        stamp_xor(new_x - w / 2, new_y - h / 2);
	      else
	        rect_xor(new_x - w / 2, new_y - h / 2,
		         new_x + w / 2, new_y + h / 2);
#else
	      rect_xor(new_x - w / 2, new_y - h / 2,
		       new_x + w / 2, new_y + h / 2);
#endif
	      update_screen(new_x - w / 2 + 96, new_y - h / 2,
		            new_x + w / 2 + 96, new_y + h / 2);
	    }
	  }
	else if (cur_tool == TOOL_SHAPES &&
		 shape_tool_mode == SHAPE_TOOL_MODE_ROTATE)
	{
	  do_shape(shape_ctr_x, shape_ctr_y,
	           shape_outer_x, shape_outer_y,
		   rotation(shape_ctr_x, shape_ctr_y,
			    old_x, old_y), 0);
		
	  do_shape(shape_ctr_x, shape_ctr_y,
	           shape_outer_x, shape_outer_y,
		   rotation(shape_ctr_x, shape_ctr_y,
			    new_x, new_y), 0);
	
	
	  /* FIXME: Do something less intensive! */

	  SDL_Flip(screen);
	}
	
	old_x = new_x;
	old_y = new_y;
      }
  }
  while (!done);
}


/* Draw using the current brush: */

void brush_draw(int x1, int y1, int x2, int y2, int update)
{
  int dx, dy, y;
  int orig_x1, orig_y1, orig_x2, orig_y2, tmp;
  float m, b;

  orig_x1 = x1;
  orig_y1 = y1;

  orig_x2 = x2;
  orig_y2 = y2;


  x1 = x1 - (img_brushes[cur_brush]->w / 2);
  y1 = y1 - (img_brushes[cur_brush]->h / 2);

  x2 = x2 - (img_brushes[cur_brush]->w / 2);
  y2 = y2 - (img_brushes[cur_brush]->h / 2);


  dx = x2 - x1;
  dy = y2 - y1;

  if (dx != 0)
    {
      m = ((float) dy) / ((float) dx);
      b = y1 - m * x1;

      if (x2 >= x1)
	dx = 1;
      else
	dx = -1;


      while (x1 != x2)
	{
	  y1 = m * x1 + b;
	  y2 = m * (x1 + dx) + b;

	  if (y1 > y2)
	    {
	      y = y1;
	      y1 = y2;
	      y2 = y;
	    }
	
	  for (y = y1; y <= y2; y++)
	    blit_brush(x1, y);
	
	  x1 = x1 + dx;
	}
    }
  else
    {
      if (y1 > y2)
	{
	  y = y1;
	  y1 = y2;
	  y2 = y;
	}

      for (y = y1; y <= y2; y++)
	blit_brush(x1, y);
    }

  if (orig_x1 > orig_x2)
  {
    tmp = orig_x1;
    orig_x1 = orig_x2;
    orig_x2 = tmp;
  }

  if (orig_y1 > orig_y2)
  {
    tmp = orig_y1;
    orig_y1 = orig_y2;
    orig_y2 = tmp;
  }


  if (update)
  {
    update_canvas(orig_x1 - (img_brushes[cur_brush]->w / 2),
		  orig_y1 - (img_brushes[cur_brush]->h / 2),
		  orig_x2 + (img_brushes[cur_brush]->w / 2),
		  orig_y2 + (img_brushes[cur_brush]->h / 2));
  }
}


/* Draw the current brush in the current color: */

void blit_brush(int x, int y)
{
  SDL_Rect dest;

  brush_counter++;

  if (brush_counter >= (img_cur_brush->h / 4))
    {
      brush_counter = 0;

      dest.x = x;
      dest.y = y;

      SDL_BlitSurface(img_cur_brush, NULL, canvas, &dest);
    }
}


/* Draw using the current stamp: */

void stamp_draw(int x, int y)
{
  SDL_Rect dest;
  SDL_Surface * tmp_surf;
  Uint32 amask;
  Uint8 r, g, b, a;
  int xx, yy;
  float col_hue, col_sat, col_val,
    stamp_hue, stamp_sat, stamp_val;
  

  dest.x = x - (img_stamps[cur_stamp]->w / 2);
  dest.y = y - (img_stamps[cur_stamp]->h / 2);

  if (stamp_colorable(cur_stamp))
  {
    /* Render the stamp in the chosen color: */

    /* FIXME: It sucks to render this EVERY TIME.  Why not just when
       they pick the color, or pick the stamp, like with brushes?
       (Why? Because I'm LAZY! :^) ) */

	  
    amask = ~(img_stamps[cur_stamp]->format->Rmask |
	      img_stamps[cur_stamp]->format->Gmask |
	      img_stamps[cur_stamp]->format->Bmask);

    tmp_surf =
      SDL_CreateRGBSurface(SDL_SWSURFACE,
			   img_stamps[cur_stamp]->w,
			   img_stamps[cur_stamp]->h,
			   img_stamps[cur_stamp]->format->BitsPerPixel,
			   img_stamps[cur_stamp]->format->Rmask,
			   img_stamps[cur_stamp]->format->Gmask,
			   img_stamps[cur_stamp]->format->Bmask,
			   amask);

    if (tmp_surf == NULL)
    {
      fprintf(stderr, "\nError: Can't render the colored stamp!\n"
		      "The Simple DirectMedia Layer error that occured was:\n"
		      "%s\n\n", SDL_GetError());

      cleanup();
      exit(1);
    }


    /* Render the stamp: */

    SDL_LockSurface(img_stamps[cur_stamp]);
    SDL_LockSurface(tmp_surf);

    for (yy = 0; yy < img_stamps[cur_stamp]->h; yy++)
    {
      for (xx = 0; xx < img_stamps[cur_stamp]->w; xx++)
      {
	SDL_GetRGBA(getpixel(img_stamps[cur_stamp], xx, yy),
		    img_stamps[cur_stamp]->format,
	            &r, &g, &b, &a);
	
	putpixel(tmp_surf, xx, yy,
		 SDL_MapRGBA(tmp_surf->format,
			     color_hexes[cur_color][0],
			     color_hexes[cur_color][1],
			     color_hexes[cur_color][2],
			     a));
      }
    }

    SDL_UnlockSurface(tmp_surf);
    SDL_UnlockSurface(img_stamps[cur_stamp]);
    
    
    /* And blit it! */
    
    SDL_BlitSurface(tmp_surf, NULL, canvas, &dest);
    SDL_FreeSurface(tmp_surf);
  }
  else if (stamp_tintable(cur_stamp))
  {
    /* Render the stamp in the chosen color: */

    /* FIXME: It sucks to render this EVERY TIME.  Why not just when
       they pick the color, or pick the stamp, like with brushes?
       (Why? Because I'm LAZY! :^) ) */

	  
    amask = ~(img_stamps[cur_stamp]->format->Rmask |
	      img_stamps[cur_stamp]->format->Gmask |
	      img_stamps[cur_stamp]->format->Bmask);

    tmp_surf =
      SDL_CreateRGBSurface(SDL_SWSURFACE,
			   img_stamps[cur_stamp]->w,
			   img_stamps[cur_stamp]->h,
			   img_stamps[cur_stamp]->format->BitsPerPixel,
			   img_stamps[cur_stamp]->format->Rmask,
			   img_stamps[cur_stamp]->format->Gmask,
			   img_stamps[cur_stamp]->format->Bmask,
			   amask);

    if (tmp_surf == NULL)
    {
      fprintf(stderr, "\nError: Can't render the colored stamp!\n"
		      "The Simple DirectMedia Layer error that occured was:\n"
		      "%s\n\n", SDL_GetError());

      cleanup();
      exit(1);
    }

    
    rgbtohsv(color_hexes[cur_color][0],
	     color_hexes[cur_color][1],
	     color_hexes[cur_color][2],
	     &col_hue, &col_sat, &col_val);
        

    /* Render the stamp: */

    SDL_LockSurface(img_stamps[cur_stamp]);
    SDL_LockSurface(tmp_surf);

    for (yy = 0; yy < img_stamps[cur_stamp]->h; yy++)
    {
      for (xx = 0; xx < img_stamps[cur_stamp]->w; xx++)
      {
	SDL_GetRGBA(getpixel(img_stamps[cur_stamp], xx, yy),
		    img_stamps[cur_stamp]->format,
	            &r, &g, &b, &a);
	
	rgbtohsv(r, g, b, &stamp_hue, &stamp_sat, &stamp_val);	

	hsvtorgb(col_hue, col_sat, stamp_val, &r, &g, &b);
	
	
	putpixel(tmp_surf, xx, yy,
		 SDL_MapRGBA(tmp_surf->format, r, g, b, a));
      }
    }

    SDL_UnlockSurface(tmp_surf);
    SDL_UnlockSurface(img_stamps[cur_stamp]);
    
    
    /* And blit it! */
    
    SDL_BlitSurface(tmp_surf, NULL, canvas, &dest);
    SDL_FreeSurface(tmp_surf);
  }
  else
  {
    /* Not a colorized stamp.  Just draw it! */
	  
    SDL_BlitSurface(img_stamps[cur_stamp], NULL,
		    canvas, &dest);
  }

  update_canvas(x - (img_stamps[cur_stamp]->w / 2),
		y - (img_stamps[cur_stamp]->h / 2),
		x + (img_stamps[cur_stamp]->w / 2),
		y + (img_stamps[cur_stamp]->h / 2));
}


/* Draw using the current brush: */

void magic_draw(int x1, int y1, int x2, int y2)
{
  int dx, dy, y;
  int orig_x1, orig_y1, orig_x2, orig_y2, tmp;
  float m, b;

  if (cur_magic == MAGIC_RAINBOW)
    rainbow_color = (rainbow_color + 1) % NUM_RAINBOW_COLORS;

  orig_x1 = x1;
  orig_y1 = y1;

  orig_x2 = x2;
  orig_y2 = y2;


  dx = x2 - x1;
  dy = y2 - y1;

  if (dx != 0)
    {
      m = ((float) dy) / ((float) dx);
      b = y1 - m * x1;

      if (x2 >= x1)
	dx = 1;
      else
	dx = -1;


      while (x1 != x2)
	{
	  y1 = m * x1 + b;
	  y2 = m * (x1 + dx) + b;

          if (y1 > y2)
          {
            for (y = y1; y >= y2; y--)
	      blit_magic(x1, y, x1, y - 1);
          }
          else
          {
            for (y = y1; y <= y2; y++)
	      blit_magic(x1, y, x1 + dx, y + 1);
          }
	
	  x1 = x1 + dx;
	}
    }
  else
    {
      if (y1 > y2)
      {
        for (y = y1; y >= y2; y--)
	  blit_magic(x1, y, x1, y - 1);
      }
      else
      {
        for (y = y1; y <= y2; y++)
	  blit_magic(x1, y, x1, y + 1);
      }
    }

  if (orig_x1 > orig_x2)
  {
    tmp = orig_x1;
    orig_x1 = orig_x2;
    orig_x2 = tmp;
  }

  if (orig_y1 > orig_y2)
  {
    tmp = orig_y1;
    orig_y1 = orig_y2;
    orig_y2 = tmp;
  }


  /* Play sound: */

  if (cur_magic == MAGIC_DRIP)
    playsound(0, SND_DRIP, 0);
  else if (cur_magic == MAGIC_CHALK)
    playsound(0, SND_CHALK, 0);
  else if (cur_magic == MAGIC_SPARKLES)
    playsound(0, SND_SPARKLES1 + (rand() % 2), 0);
  else if (cur_magic == MAGIC_FLIP)
    playsound(0, SND_FLIP, 0);
  else if (cur_magic == MAGIC_MIRROR)
    playsound(0, SND_MIRROR, 0);
  else if (cur_magic == MAGIC_NEGATIVE)
    playsound(0, SND_NEGATIVE, 0);
  else if (cur_magic == MAGIC_BLUR)
    playsound(0, SND_BLUR, 0);
  else if (cur_magic == MAGIC_THICK)
    playsound(0, SND_THICK, 0);
  else if (cur_magic == MAGIC_THIN)
    playsound(0, SND_THIN, 0);
  else if (cur_magic == MAGIC_BLOCKS && ((rand() % 10) < 5))
    playsound(0, SND_BLOCKS, 0);
  else if (cur_magic == MAGIC_FADE)
    playsound(0, SND_FADE, 0);
  else if (cur_magic == MAGIC_RAINBOW)
    playsound(0, SND_RAINBOW, 0);


  /* FIXME: Arbitrary? */

  update_canvas(orig_x1 - 32, orig_y1 - 32,
		orig_x2 + 32, orig_y2 + 32);
}


/* Draw the current brush in the current color: */

void blit_magic(int x, int y, int x2, int y2)
{
  int xx, yy, w, h;
  Uint32 colr;
  Uint8 r, g, b,
        r1, g1, b1,
        r2, g2, b2,
	r3, g3, b3,
	r4, g4, b4;
  SDL_Surface * last;
  SDL_Rect src, dest;


  /* In case we need to use the current canvas (just saved to undo buf)... */

  if (cur_undo > 0)
    last = undo_bufs[cur_undo - 1];
  else
    last = undo_bufs[NUM_UNDO_BUFS - 1];


  brush_counter++;

  if (brush_counter >= 4)  /* FIXME: Arbitrary? */
    {
      brush_counter = 0;


      if (cur_magic == MAGIC_BLUR)
      {
	/* FIXME: Circular "brush?" */

	SDL_LockSurface(canvas);
	for (yy = y - 16; yy < y + 16; yy = yy + 2)
	{
          for (xx = x - 16; xx < x + 16; xx = xx + 2)
	  {
            SDL_GetRGB(getpixel(canvas, clamp(0, xx, canvas->w - 1),
				        clamp(0, yy - 1, canvas->h - 1)),
		       canvas->format,
		       &r1, &g1, &b1);
	
            SDL_GetRGB(getpixel(canvas, clamp(0, xx - 1, canvas->w - 1),
				        clamp(0, yy, canvas->h - 1)),
		       canvas->format,
		       &r2, &g2, &b2);
	
            SDL_GetRGB(getpixel(canvas, clamp(0, xx + 1, canvas->w - 1),
				        clamp(0, yy, canvas->h - 1)),
		       canvas->format,
		       &r3, &g3, &b3);
	
            SDL_GetRGB(getpixel(canvas, clamp(0, xx, canvas->w - 1),
				        clamp(0, yy + 1, canvas->h - 1)),
		       canvas->format,
		       &r4, &g4, &b4);
	

	    r = (r1 + r2 + r3 + r4) >> 2;
	    g = (g1 + g2 + g3 + g4) >> 2;
	    b = (b1 + b2 + b3 + b4) >> 2;

	    putpixel(canvas, xx, yy,
		     SDL_MapRGB(canvas->format, r, g, b));
	  }
	}
	SDL_UnlockSurface(canvas);
      }
      else if (cur_magic == MAGIC_BLOCKS)
      {
	/* Put x/y on exact grid points: */

	x = (x / 4) * 4;
	y = (y / 4) * 4;

	SDL_LockSurface(last);
	SDL_LockSurface(canvas);

	for (yy = y - 8; yy < y + 8; yy = yy + 4)
	{
	  for (xx = x - 8; xx < x + 8; xx = xx + 4)
	  {
            /* Get average color around here: */
		
	    SDL_GetRGB(getpixel(last, xx, yy), last->format,
		       &r1, &g1, &b1);
	
	    SDL_GetRGB(getpixel(last, xx + 2, yy), last->format,
		       &r2, &g2, &b2);
	
	    SDL_GetRGB(getpixel(last, xx, yy + 2), last->format,
		       &r3, &g3, &b3);
	
	    SDL_GetRGB(getpixel(last, xx + 2, yy + 2), last->format,
		       &r4, &g4, &b4);

	    r = (r1 + r2 + r3 + r4) / 4;
	    g = (g1 + g2 + g3 + g4) / 4;
	    b = (b1 + b2 + b3 + b4) / 4;

	
	    /* Draw block: */

	    dest.x = xx;
	    dest.y = yy;
	    dest.w = 4;
	    dest.h = 4;

	    SDL_FillRect(canvas, &dest, SDL_MapRGB(canvas->format, r, g, b));
	  }
	}
	
	SDL_UnlockSurface(canvas);
	SDL_UnlockSurface(last);
      }
      else if (cur_magic == MAGIC_NEGATIVE)
      {
	SDL_LockSurface(last);
	SDL_LockSurface(canvas);

	for (yy = y - 16; yy < y + 16; yy++)
	{
	  for (xx = x - 16; xx < x + 16; xx++)
	  {
            /* Get average color around here: */
		
	    SDL_GetRGB(getpixel(last, xx, yy), last->format,
		       &r, &g, &b);
	
	    r = 0xFF - r;
	    g = 0xFF - g;
	    b = 0xFF - b;

	    putpixel(canvas, xx, yy, SDL_MapRGB(canvas->format, r, g, b));
	  }
	}

	SDL_UnlockSurface(canvas);
	SDL_UnlockSurface(last);
      }
      else if (cur_magic == MAGIC_FADE)
      {
	SDL_LockSurface(last);
	SDL_LockSurface(canvas);

	for (yy = y - 16; yy < y + 16; yy++)
	{
	  for (xx = x - 16; xx < x + 16; xx++)
	  {
            /* Get average color around here: */
		
	    SDL_GetRGB(getpixel(last, xx, yy), last->format,
		       &r, &g, &b);
	
	    r = min(r + 48, 255);
	    g = min(g + 48, 255);
	    b = min(b + 48, 255);

	    putpixel(canvas, xx, yy, SDL_MapRGB(canvas->format, r, g, b));
	  }
	}

	SDL_UnlockSurface(canvas);
	SDL_UnlockSurface(last);
      }
      else if (cur_magic == MAGIC_RAINBOW)
      {
	/* Pick next color: */
	
	colr = SDL_MapRGB(canvas->format,
			  rainbow_hexes[rainbow_color][0],
			  rainbow_hexes[rainbow_color][1],
			  rainbow_hexes[rainbow_color][2]);
	

	/* Draw the shape: */

	for (yy = 0; yy <= 16; yy++)
	{
	  w = (yy * yy) / 16;


	  /* Top half: */
	
	  dest.x = x - 16 + w;
	  dest.w = 32 - (w * 2);
	  dest.y = y - yy;;
	  dest.h = 1;

	  SDL_FillRect(canvas, &dest, colr);
	

	  /* Bottom half: */

	  dest.x = x - 16 + w;
	  dest.w = 32 - (w * 2);
	  dest.y = y + yy;
	  dest.h = 1;

	  SDL_FillRect(canvas, &dest, colr);
	}
      }
      else if (cur_magic == MAGIC_CHALK)
      {
	SDL_LockSurface(last);

	for (yy = y - 8; yy <= y + 8; yy = yy + 4)
	{
	  for (xx = x - 8; xx <= x + 8; xx = xx + 4)
	  {
	    dest.x = xx + ((rand() % 5) - 2);
	    dest.y = yy + ((rand() % 5) - 2);
	    dest.w = (rand() % 4) + 2;
	    dest.h = (rand() % 4) + 2;
	
	    colr = getpixel(last, clamp(0, xx, canvas->w-1),
			    clamp(0, yy, canvas->h-1));
	    SDL_FillRect(canvas, &dest, colr);
	  }
	}

	SDL_UnlockSurface(last);
      }
      else if (cur_magic == MAGIC_DRIP)
      {
	for (xx = x - 8; xx <= x + 8; xx++)
	{
	  h = (rand() % 8) + 8;

	  for (yy = y; yy <= y + h; yy++)
	  {
	    src.x = xx;
	    src.y = y;
	    src.w = 1;
	    src.h = 16;

	    dest.x = xx;
	    dest.y = yy;
	
	    SDL_BlitSurface(last, &src, canvas, &dest);
	  }
	}
      }
      else if (cur_magic == MAGIC_SPARKLES)
      {
	if ((rand() % 10) < 2)
	{
	  src.x = 0;
	  src.y = (rand() % 4) * 32;
	  src.w = 32;
	  src.h = 32;

	  dest.x = x - 16;
	  dest.y = y - 16;

	  SDL_BlitSurface(img_sparkles, &src, canvas, &dest);
	}
      }
      else if (cur_magic == MAGIC_FLIP)
      {
	for (yy = 0; yy < canvas->h; yy++)
	{
	  src.x = 0;
	  src.y = yy;
	  src.w = canvas->w;
	  src.h = 1;

	  dest.x = 0;
	  dest.y = canvas->h - yy - 1;
	
	  SDL_BlitSurface(last, &src, canvas, &dest);
	}
      }
      else if (cur_magic == MAGIC_MIRROR)
      {
	for (xx = 0; xx < canvas->w; xx++)
	{
	  src.x = xx;
	  src.y = 0;
	  src.w = 1;
	  src.h = canvas->h;

	  dest.x = canvas->w - xx - 1;
	  dest.y = 0;
	
	  SDL_BlitSurface(last, &src, canvas, &dest);
	}
      }
      else if (cur_magic == MAGIC_THIN || cur_magic == MAGIC_THICK)
      {
	SDL_LockSurface(last);
	SDL_LockSurface(canvas);

	for (xx = -8; xx <= 8; xx++)
	{
	  for (yy = -8; yy <= 8; yy++)
	  {
	    SDL_GetRGB(getpixel(last, x + xx, y + yy), last->format,
		       &r, &g, &b);

	    r = min(r, 255);
	    g = min(g, 255);
	    b = min(b, 255);
	
	    if ((cur_magic == MAGIC_THIN && (((r + g + b) / 3) > 128)) ||
	        (cur_magic == MAGIC_THICK && (((r + g + b) / 3) <= 128)))
	    {
	      putpixel(canvas, x + xx + 0, y + yy - 1,
		       SDL_MapRGB(canvas->format, r, g, b));
	      putpixel(canvas, x + xx - 1, y + yy + 0,
		       SDL_MapRGB(canvas->format, r, g, b));
	      putpixel(canvas, x + xx + 1, y + yy + 0,
		       SDL_MapRGB(canvas->format, r, g, b));
	      putpixel(canvas, x + xx + 0, y + yy + 1,
		       SDL_MapRGB(canvas->format, r, g, b));
	    }
	  }
	}
	
	SDL_UnlockSurface(canvas);
	SDL_UnlockSurface(last);
      }
    }
}


/* Store canvas into undo buffer: */

void rec_undo_buffer(void)
{
  int wanna_update_toolbar;

  wanna_update_toolbar = 0;


  SDL_BlitSurface(canvas, NULL, undo_bufs[cur_undo], NULL);

  cur_undo = (cur_undo + 1) % NUM_UNDO_BUFS;

  if (cur_undo == oldest_undo)
    oldest_undo = (oldest_undo + 1) % NUM_UNDO_BUFS;

  newest_undo = cur_undo;

#ifdef DEBUG
  printf("DRAW: Current=%d  Oldest=%d  Newest=%d\n",
	 cur_undo, oldest_undo, newest_undo);
#endif


  /* Update toolbar buttons, if needed: */

  if (tool_avail[TOOL_UNDO] == 0)
    {
      tool_avail[TOOL_UNDO] = 1;
      wanna_update_toolbar = 1;
    }

  if (tool_avail[TOOL_REDO])
    {
      tool_avail[TOOL_REDO] = 0;
      wanna_update_toolbar = 1;
    }

  if (wanna_update_toolbar)
    {
      draw_toolbar();
      SDL_UpdateRect(screen, 0, 0, 96, (48 * 7) + 40);
    }
}


/* Update the screen with the new canvas: */

void update_canvas(int x1, int y1, int x2, int y2)
{
  SDL_Rect dest;

  dest.x = 96;
  dest.y = 0;
  SDL_BlitSurface(canvas, NULL, screen, &dest);
  update_screen(x1 + 96, y1, x2 + 96, y2);
}


/* Show program version: */

void show_version(void)
{
  printf("\nTux Paint\n");
  printf("  Version " VER_VERSION " (" VER_DATE ")\n");

#ifdef LOW_QUALITY_THUMBNAILS
  printf("  Low Quality Thumbnails enabled\n");
#endif

#ifdef LOW_QUALITY_COLOR_SELECTOR
  printf("  Low Quality Color Selector enabled\n");
#endif

#ifdef NOSOUND
  printf("  Sound disabled\n");
#endif

  printf("\n");
}


/* Show usage display: */

void show_usage(FILE * f, char * prg)
{
  fprintf(f,
    "Usage: %s {--usage | --help | --version | --copying}\n"
    "       %s [--fullscreen] [--nosound] [--noquit] [--simpleshapes]\n"
    "          [--lang LANGUAGE | --locale LOCALE]\n\n"
    "LANGUAGE may be one of:\n"
    "  english\n"
    "  dutch\n"
    "  french       francais\n"
    "  german       deutsch\n"
    "  italian      italiano\n"
    "  finnish      suomi\n"
    "  norwegian    nynorsk      norsk\n"
    "  turkish\n\n",
    prg, prg);
}


const char *getfilename(const char* path)
{
    char    *p;

    if ( (p = strrchr( path, '\\' )) != NULL )
              return p+1;
    if ( (p = strrchr( path, '/' )) != NULL )
              return p+1;
      return path;
}


/* Setup: */

void setup(int argc, char * argv[])
{
  int i;
  SDL_Rect dest;
  char str[128];
  FILE * fi;
  SDL_Color black = {0, 0, 0, 0};
  SDL_Surface * tmp_surf;
#ifndef LOW_QUALITY_COLOR_SELECTOR
  int x, y;
  SDL_Surface * tmp_btn;
  Uint8 r, g, b, a;
#endif


  /* Set default options: */

  use_sound = 1;
  fullscreen = 0;
  simple_shapes = 0;
  disable_quit = 0;


  /* Load options from .rc file: */

  if (getenv("HOME") != NULL)
  {
    /* Should it be "~/.tuxpaint/tuxpaintrc" instead???
       Comments welcome ... bill@newbreedsoftware.com */

    snprintf(str, sizeof(str), "%s/.tuxpaintrc", getenv("HOME"));
  }
  else
  {
    strcpy(str, "tuxpaint.cfg");
  }

  fi = fopen(str, "r");
  if (fi != NULL)
  {
    do
    {
      fgets(str, sizeof(str), fi);

      if (!feof(fi))
      {
	debug(str);
	
	
	/* Should "lang=" and "locale=" be here as well???
           Comments welcome ... bill@newbreedsoftware.com */


	/* FIXME: This should be handled better! */
	/* (e.g., complain on illegal lines, support comments, blanks, etc.) */

        if (strstr(str, "fullscreen=yes") != 0)
	{
	  fullscreen = 1;
	}
	else if (strstr(str, "noquit=yes") != 0)
	{
	  disable_quit = 1;
	}
	else if (strstr(str, "nosound=yes") != 0)
	{
	  use_sound = 0;
	}
	else if (strstr(str, "simpleshapes=yes") != 0)
	{
	  simple_shapes = 1;
	}
      }
    }
    while (!feof(fi));

    fclose(fi);
  }
  else
  {
    debug(str);
  }


  /* Handle command-line arguments: */

  for (i = 1; i < argc; i++)
  {
    if (strcmp(argv[i], "--fullscreen") == 0 || strcmp(argv[i], "-f") == 0)
    {
      fullscreen = 1;
    }
    else if (strcmp(argv[i], "--windowed") == 0 || strcmp(argv[i], "-w") == 0)
    {
      fullscreen = 0;
    }
    else if (strcmp(argv[i], "--simpleshapes") == 0 ||
	     strcmp(argv[i], "-s") == 0)
    {
      simple_shapes = 1;
    }
    else if (strcmp(argv[i], "--complexshapes") == 0)
    {
      simple_shapes = 0;
    }
    else if (strcmp(argv[i], "--noquit") == 0 || strcmp(argv[i], "-x") == 0)
    {
      disable_quit = 1;
    }
    else if (strcmp(argv[i], "--quit") == 0)
    {
      disable_quit = 0;
    }
    else if (strcmp(argv[i], "--nosound") == 0 || strcmp(argv[i], "-q") == 0)
    {
      use_sound = 0;
    }
    else if (strcmp(argv[i], "--sound") == 0)
    {
      use_sound = 1;
    }
    else if (strcmp(argv[i], "--locale") == 0 || strcmp(argv[i], "-L") == 0)
    {
      if (i < argc - 1)
      {
	snprintf(str, sizeof(str), "LANG=%s", argv[i + 1]);
	putenv(str);
	
	setlocale(LC_ALL, "");  /* argv[i + 1] */
	i++;
      }
      else
      {
	/* Forgot to specify the language! */
	
	fprintf(stderr, "%s takes an argument\n", argv[i]);
        show_usage(stderr, (char *) getfilename(argv[0]));
        exit(1);
      }
    }
    else if (strcmp(argv[i], "--lang") == 0 || strcmp(argv[i], "-l") == 0)
    {
      if (i < argc - 1)
      {
	if (strcmp(argv[i + 1], "english") == 0)
  	{
	  putenv("LANG=C");
	}
	else if (strcmp(argv[i + 1], "spanish") == 0 ||
	    strcmp(argv[i + 1], "espanol") == 0)
  	{
	  putenv("LANG=es_ES@euro");
	}
	else if (strcmp(argv[i + 1], "french") == 0 ||
	    strcmp(argv[i + 1], "francais") == 0)
  	{
	  putenv("LANG=fr_FR@euro");
	}
	else if (strcmp(argv[i + 1], "german") == 0 ||
	    strcmp(argv[i + 1], "deutsch") == 0)
  	{
	  putenv("LANG=de_DE@euro");
	}
	else if (strcmp(argv[i + 1], "finnish") == 0 ||
		 strcmp(argv[i + 1], "suomi") == 0)
  	{
	  putenv("LANG=fi_FI@euro");
	}
	else if (strcmp(argv[i + 1], "turkish") == 0)
	{
	  putenv("LANG=tr_TR@euro");
	}
	else if (strcmp(argv[i + 1], "norwegian") == 0 ||
		 strcmp(argv[i + 1], "nynorsk") == 0 ||
		 strcmp(argv[i + 1], "norsk") == 0)
  	{
	  putenv("LANG=nn_NO");
	}
	else if (strcmp(argv[i + 1], "italian") == 0 ||
		 strcmp(argv[i + 1], "italiano") == 0)
  	{
	  putenv("LANG=it_IT@euro");
	}
	else if (strcmp(argv[i + 1], "dutch") == 0)
  	{
	  putenv("LANG=nl_NL@euro");
	}
	else
	{
	  fprintf(stderr, "%s is an invalid language\n", argv[i + 1]);
          show_usage(stderr, (char *) getfilename(argv[0]));
          exit(1);
	}
	
	setlocale(LC_ALL, "");
	i++;
      }
      else
      {
	/* Forgot to specify the language! */
	
	fprintf(stderr, "%s takes an argument\n", argv[i]);
        show_usage(stderr, (char *) getfilename(argv[0]));
        exit(1);
      }
    }
    else if (strcmp(argv[i], "--version") == 0 || strcmp(argv[i], "-v") == 0)
    {
      show_version();
      exit(0);
    }
    else if (strcmp(argv[i], "--copying") == 0 || strcmp(argv[i], "-c") == 0)
    {
      show_version();
      printf(
        "\n"
        "This program is free software; you can redistribute it\n"
        "and/or modify it under the terms of the GNU General Public\n"
        "License as published by the Free Software Foundation;\n"
        "either version 2 of the License, or (at your option) any\n"
        "later version.\n"
        "\n"
        "This program is distributed in the hope that it will be\n"
        "useful and entertaining, but WITHOUT ANY WARRANTY; without\n"
        "even the implied warranty of MERCHANTABILITY or FITNESS\n"
        "FOR A PARTICULAR PURPOSE.  See the GNU General Public\n"
        "License for more details.\n"
        "\n"
        "You should have received a copy of the GNU General Public\n"
        "License along with this program; if not, write to the Free\n"
        "Software Foundation, Inc., 59 Temple Place, Suite 330,\n"
        "Boston, MA  02111-1307  USA\n"
        "\n"
      );
      exit(0);
    }
    else if (strcmp(argv[i], "--help") == 0 ||
	     strcmp(argv[i], "-h") == 0)
    {
      printf("\n");
      show_version();
      printf("\n");
      show_usage(stdout, (char *) getfilename(argv[0]));

      printf(
	"\n"
	"See: " DOC_PREFIX "README.txt\n"
	"\n");
      exit(0);
    }
    else if (strcmp(argv[i], "--usage") == 0 ||
	     strcmp(argv[i], "-u") == 0)
    {
      show_usage(stdout, (char *) getfilename(argv[0]));
      exit(0);
    }
    else
    {
      show_usage(stderr, (char *) getfilename(argv[0]));
      exit(1);
    }
  }


  /* Set up language: */
  
  bindtextdomain("tuxpaint", LOCALEDIR);

  textdomain("tuxpaint");

  language = current_language();


#ifndef WIN32
  putenv("SDL_VIDEO_X11_WMCLASS=TuxPaint.TuxPaint");
#endif


  /* Init SDL Video: */

  if (SDL_Init(SDL_INIT_VIDEO) < 0)
  {
    fprintf(stderr,
	    "\nError: I could not initialize video!\n"
	    "The Simple DirectMedia Layer error that occured was:\n"
	    "%s\n\n", SDL_GetError());
    exit(1);
  }


  /* Init TTF stuff: */

  if (TTF_Init() < 0)
  {
    fprintf(stderr,
            "\nError: I could not initialize the font (TTF) library!\n"
	    "The Simple DirectMedia Layer error that occured was:\n"
	    "%s\n\n", SDL_GetError());

    SDL_Quit();
    exit(1);
  }


  /* Init SDL Audio and set-up Mixer: */

#ifndef NOSOUND
  if (use_sound)
  {
    if (SDL_Init(SDL_INIT_AUDIO) < 0)
    {
      fprintf(stderr,
	      "\nWarning: I could not initialize audio!\n"
	      "The Simple DirectMedia Layer error that occured was:\n"
	      "%s\n\n", SDL_GetError());
      use_sound = 0;
    }
    else
    {
      if (Mix_OpenAudio(44100, AUDIO_S16, 2, 256) < 0)
      {
	fprintf(stderr,
	        "\nWarning: I could not set up audio for 44100 Hz "
		"16-bit stereo.\n"
		"The Simple DirectMedia Layer error that occured was:\n"
		              "%s\n\n", SDL_GetError());
	use_sound = 0;
      }
      else
      {
	/* Load sounds: */

	for (i = 0; i < NUM_SOUNDS; i++)
	{
          sounds[i] = Mix_LoadWAV(sound_fnames[i]);
	
	  if (sounds[i] == NULL)
	  {
            fprintf(stderr,
	            "\nWarning: I couldn't open a sound file:\n%s\n"
		    "The Simple DirectMedia Layer error that occured was:\n"
		    "%s\n\n", sound_fnames[i], SDL_GetError());
            use_sound = 0;
	  }
	}
      }
    }
  }
#endif


  /* Open Window: */

  if (fullscreen)
  {
    screen = SDL_SetVideoMode(640, 480, 16, SDL_FULLSCREEN | SDL_HWSURFACE);

    if (screen == NULL)
    {
      fprintf(stderr,
             "\nWarning: I could not open the display in fullscreen mode.\n"
             "The Simple DirectMedia Layer error that occured was:\n"
             "%s\n\n", SDL_GetError());

     fullscreen = 0;
    }
  }


  if (!fullscreen)
    screen = SDL_SetVideoMode(640, 480, 16, SDL_HWSURFACE);

  if (screen == NULL)
  {
    fprintf(stderr,
	    "\nError: I could not open the display.\n"
	    "The Simple DirectMedia Layer error that occured was:\n"
            "%s\n\n", SDL_GetError());

    cleanup();
    exit(1);
  }


  /* Set window icon and caption: */

  seticon();
  SDL_WM_SetCaption("Tux Paint", "Tux Paint");


  /* Create drawing canvas: */

  canvas = SDL_CreateRGBSurface(screen->flags,
				640 - (96 * 2), (48 * 7) + 40,
				screen->format->BitsPerPixel,
				screen->format->Rmask,
				screen->format->Gmask,
				screen->format->Bmask,
				0);


  if (canvas == NULL)
  {
    fprintf(stderr, "\nError: Can't build drawing canvas!\n"
		    "The Simple DirectMedia Layer error that occured was:\n"
		    "%s\n\n", SDL_GetError());

    cleanup();
    exit(1);
  }

  SDL_FillRect(canvas, NULL, SDL_MapRGB(canvas->format, 255, 255, 255));


  /* Create undo buffer space: */

  for (i = 0; i < NUM_UNDO_BUFS; i++)
    {
      undo_bufs[i] = SDL_CreateRGBSurface(screen->flags,
					  640 - (96 * 2), (48 * 7) + 40,
					  screen->format->BitsPerPixel,
					  screen->format->Rmask,
					  screen->format->Gmask,
					  screen->format->Bmask,
					  0);


      if (undo_bufs[i] == NULL)
	{
	  fprintf(stderr, "\nError: Can't build undo buffer! (%d of %d)\n"
		  "The Simple DirectMedia Layer error that occured was:\n"
		  "%s\n\n", i + 1, NUM_UNDO_BUFS, SDL_GetError());
	
	  cleanup();
	  exit(1);
	}
    }


  /* Load and display title image: */

  img_title = loadimage(DATA_PREFIX "images/title.jpg");
  img_loading = loadimage(DATA_PREFIX "images/ui/loading.png");
  img_progress = loadimage(DATA_PREFIX "images/ui/progress.png");

  SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 255, 255, 255));

  SDL_BlitSurface(img_title, NULL, screen, NULL);

  dest.x = (640 - img_loading->w) / 2;
  dest.y = (480 - img_loading->h) / 2;
  SDL_BlitSurface(img_loading, NULL, screen, &dest);
  
  prog_bar_ctr = 0;
  show_progress_bar();

  SDL_Flip(screen);


  /* Load other images: */

  for (i = 0; i < NUM_TOOLS; i++)
    img_tools[i] = loadimage(tool_img_fnames[i]);

  for (i = 0; i < NUM_TITLES; i++)
    img_titles[i] = loadimage(title_img_fnames[i]);

  img_btn_up = loadimage(DATA_PREFIX "images/ui/btn_up.png");
  img_btn_down = loadimage(DATA_PREFIX "images/ui/btn_down.png");
  img_btn_off = loadimage(DATA_PREFIX "images/ui/btn_off.png");

  show_progress_bar();

  img_yes = loadimage(DATA_PREFIX "images/ui/yes.png");
  img_no = loadimage(DATA_PREFIX "images/ui/no.png");

  img_open = loadimage(DATA_PREFIX "images/ui/open.png");
  img_erase = loadimage(DATA_PREFIX "images/ui/erase.png");
  img_back = loadimage(DATA_PREFIX "images/ui/back.png");

  show_progress_bar();

  img_cursor_up = loadimage(DATA_PREFIX "images/ui/cursor_up.png");
  img_cursor_down = loadimage(DATA_PREFIX "images/ui/cursor_down.png");

  img_scroll_up = loadimage(DATA_PREFIX "images/ui/scroll_up.png");
  img_scroll_down = loadimage(DATA_PREFIX "images/ui/scroll_down.png");

  img_scroll_up_off = loadimage(DATA_PREFIX "images/ui/scroll_up_off.png");
  img_scroll_down_off = loadimage(DATA_PREFIX "images/ui/scroll_down_off.png");

  img_paintcan = loadimage(DATA_PREFIX "images/ui/paintcan.png");

  show_progress_bar();

  img_sparkles = loadimage(DATA_PREFIX "images/ui/sparkles.png");


  /* Load brushes: */

#ifndef NOSOUND
  loadarbitrary(img_brushes, NULL, NULL, NULL, &num_brushes, 0,
	        MAX_BRUSHES, DATA_PREFIX "brushes");
#else
  loadarbitrary(img_brushes, NULL, NULL, &num_brushes, 0,
	        MAX_BRUSHES, DATA_PREFIX "brushes");
#endif

  if (num_brushes == 0)
  {
    fprintf(stderr,
            "\nError: No brushes found in " DATA_PREFIX "brushes/\n\n");
    cleanup();
    exit(1);
  }


  /* Load system fonts: */

  font = TTF_OpenFont(DATA_PREFIX "fonts/efont_serif_b.ttf", 18);

  if (font == NULL)
  {
    fprintf(stderr,
            "\nError: Can't load font file: "
	    DATA_PREFIX "fonts/efont_serif_b.ttf\n"
	    "The Simple DirectMedia Layer error that occured was:\n"
	    "%s\n\n", SDL_GetError());

    cleanup();
    exit(1);
  }


  small_font = TTF_OpenFont(DATA_PREFIX "fonts/efont_serif_b.ttf", 14);

  if (small_font == NULL)
  {
    fprintf(stderr,
            "\nError: Can't load font file: "
	    DATA_PREFIX "fonts/efont_serif_b.ttf\n"
	    "The Simple DirectMedia Layer error that occured was:\n"
	    "%s\n\n", SDL_GetError());

    cleanup();
    exit(1);
  }


  /* Load stamps: */

#ifndef NOSOUND
  loadarbitrary(img_stamps, txt_stamps, inf_stamps, snd_stamps, &num_stamps, 0,
	        MAX_STAMPS, DATA_PREFIX "stamps");
#else
  loadarbitrary(img_stamps, txt_stamps, inf_stamps, &num_stamps, 0,
	        MAX_STAMPS, DATA_PREFIX "stamps");
#endif

  if (num_stamps == 0)
  {
    fprintf(stderr,
            "\nWarning: No stamps found in " DATA_PREFIX "stamps/\n\n");
  }


  /* Create stamp thumbnails: */

  for (i = 0; i < num_stamps; i++)
  {
    if (img_stamps[i]->w > 40 ||
	img_stamps[i]->h > 40)
    {
      img_stamp_thumbs[i] = thumbnail(img_stamps[i], 40, 40, 1);
    }
    else
    {
      img_stamp_thumbs[i] = NULL;
    }

    show_progress_bar();
  }


  /* Load magic icons: */

  for (i = 0; i < NUM_MAGICS; i++)
  {
    img_magics[i] = loadimage(magic_img_fnames[i]);
    show_progress_bar();
  }


  /* Load shape icons: */

  for (i = 0; i < NUM_SHAPES; i++)
  {
    img_shapes[i] = loadimage(shape_img_fnames[i]);
    show_progress_bar();
  }


  /* Load tip tux images: */

  for (i = 0; i < NUM_TIP_TUX; i++)
  {
    img_tux[i] = loadimage(tux_img_fnames[i]);
    show_progress_bar();
  }


  /* Generate color selection buttons: */

#ifndef LOW_QUALITY_COLOR_SELECTOR

  /* Create appropriately-shaped buttons: */

  tmp_btn = thumbnail(img_btn_down, (640 - 96) / NUM_COLORS, 48, 0);


  /* Create surfaces to draw them into: */

  for (i = 0; i < NUM_COLORS; i++)
  {

    img_color_btns[i] = SDL_CreateRGBSurface(screen->flags,
		                             /* (640 - 96) / NUM_COLORS, 48, */
		    			     tmp_btn->w, tmp_btn->h,
				             screen->format->BitsPerPixel,
				             screen->format->Rmask,
				             screen->format->Gmask,
				             screen->format->Bmask,
					     0);

    if (img_color_btns[i] == NULL)
    {
      fprintf(stderr, "\nError: Can't build color button! (%d of %d)\n"
		      "The Simple DirectMedia Layer error that occured was:\n"
                      "%s\n\n", i + 1, NUM_COLORS, SDL_GetError());

      cleanup();
      exit(1);
    }

    SDL_LockSurface(img_color_btns[i]);
  }


  /* Generate the buttons based on the thumbnails: */


  SDL_LockSurface(tmp_btn);

  for (y = 0; y < tmp_btn->h /* 48 */; y++)
  {
    for (x = 0; x < tmp_btn->w /* (640 - 96) / NUM_COLORS */; x++)
    {
      SDL_GetRGB(getpixel(tmp_btn, x, y), tmp_btn->format,
		 &r, &g, &b);
      a = 255 - ((r + g + b) / 3);

      for (i = 0; i < NUM_COLORS; i++)
      {
	putpixel(img_color_btns[i], x, y,
	         SDL_MapRGB(img_color_btns[i]->format,
			    alpha(color_hexes[i][0], 255, a),
			    alpha(color_hexes[i][1], 255, a),
			    alpha(color_hexes[i][2], 255, a)));
      }
    }
  }

  for (i = 0; i < NUM_COLORS; i++)
    SDL_UnlockSurface(img_color_btns[i]);

  SDL_UnlockSurface(tmp_btn);
  SDL_FreeSurface(tmp_btn);

#endif


  /* Creat button labels: */

  for (i = 0; i < NUM_TOOLS; i++)
  {
    tmp_surf = RENDER_TEXT(small_font, gettext(tool_names[i]), black);
    img_tool_names[i] = thumbnail(tmp_surf,
		                  min(48, tmp_surf->w), tmp_surf->h, 0);
    SDL_FreeSurface(tmp_surf);
  }

  for (i = 0; i < NUM_MAGICS; i++)
  {
    tmp_surf = RENDER_TEXT(small_font, gettext(magic_names[i]), black);
    img_magic_names[i] = thumbnail(tmp_surf,
		                   min(48, tmp_surf->w), tmp_surf->h, 0);
    SDL_FreeSurface(tmp_surf);
  }

  for (i = 0; i < NUM_SHAPES; i++)
  {
    tmp_surf = RENDER_TEXT(small_font, gettext(shape_names[i]), black);
    img_shape_names[i] = thumbnail(tmp_surf,
		                   min(48, tmp_surf->w), tmp_surf->h, 0);
    SDL_FreeSurface(tmp_surf);
  }


  /* Seed random-number generator: */

  srand(SDL_GetTicks());
}


void seticon(void)
{
  int masklen;
  Uint8 * mask;
  SDL_Surface * icon;


  /* Load icon into a surface: */

#ifndef WIN32
  icon = IMG_Load(DATA_PREFIX "images/icon.png");
#else
  icon = IMG_Load(DATA_PREFIX "images/icon-win32.png");
#endif

  if (icon == NULL)
    {
      fprintf(stderr,
	      "\nWarning: I could not load the icon image: %s\n"
	      "The Simple DirectMedia error that occured was:\n"
	      "%s\n\n", DATA_PREFIX "images/icon.png", SDL_GetError());
      return;
    }


  /* Create mask: */

  masklen = (((icon -> w) + 7) / 8) * (icon -> h);
  mask = malloc(masklen * sizeof(Uint8));
  memset(mask, 0xFF, masklen);


  /* Set icon: */

  SDL_WM_SetIcon(icon, mask);


  /* Free icon surface & mask: */

  free(mask);
  SDL_FreeSurface(icon);
}


/* Load an image: */

SDL_Surface * loadimage(char * fname)
{
  SDL_Surface * s, * disp_fmt_s;


  /* Load the image file: */

  s = IMG_Load(fname);
  if (s == NULL)
    {
      fprintf(stderr,
	      "\nError: I couldn't load a graphics file:\n"
	      "%s\n"
	      "The Simple DirectMedia Layer error that occured was:\n"
	      "%s\n\n", fname, SDL_GetError());

      cleanup();
      exit(1);
    }


  /* Convert to the display format: */

  disp_fmt_s = SDL_DisplayFormatAlpha(s);
  if (disp_fmt_s == NULL)
    {
      fprintf(stderr,
	      "\nError: I couldn't convert a graphics file:\n"
	      "%s\n"
	      "The Simple DirectMedia Layer error that occured was:\n"
	      "%s\n\n", fname, SDL_GetError());

      cleanup();
      exit(1);
    }


  /* Free the temp. surface & return the converted one: */

  SDL_FreeSurface(s);

  return(disp_fmt_s);
}


/* Draw the toolbar: */

void draw_toolbar(void)
{
  int i;
  SDL_Rect dest;


  dest.x = 0;
  dest.y = 0;
  SDL_BlitSurface(img_titles[TITLE_TOOLS], NULL, screen, &dest);

  for (i = 0; i < NUM_TOOLS; i++)
    {
      dest.x = ((i % 2) * 48);
      dest.y = ((i / 2) * 48) + 40;


      if (i == cur_tool)
      {
        SDL_BlitSurface(img_btn_down, NULL, screen, &dest);
      }
      else if (tool_avail[i])
      {
        SDL_BlitSurface(img_btn_up, NULL, screen, &dest);
      }
      else
      {
        SDL_BlitSurface(img_btn_off, NULL, screen, &dest);	
      }

      dest.x = ((i % 2) * 48) + 4;
      dest.y = ((i / 2) * 48) + 40 + 4;

      SDL_BlitSurface(img_tools[i], NULL, screen, &dest);


      dest.x = ((i % 2) * 48) + 4 + (40 - img_tool_names[i]->w) / 2;
      dest.y = ((i / 2) * 48) + 40 + 4 + (44 - img_tool_names[i]->h);

      SDL_BlitSurface(img_tool_names[i], NULL, screen, &dest);
    }
}


/* Draw magic controls: */

void draw_magic(void)
{
  int i;
  SDL_Rect dest;


  /* FIXME: Should we worry about more than 14 magic effects? :^/ */


  dest.x = 640 - 96;
  dest.y = 0;
  SDL_BlitSurface(img_titles[TITLE_MAGIC], NULL, screen, &dest);

  for (i = 0; i < 14; i++)
    {
      dest.x = 640 - 96 + ((i % 2) * 48);
      dest.y = ((i / 2) * 48) + 40;


      if (i < NUM_MAGICS)
      {
        if (i == cur_magic)
        {
          SDL_BlitSurface(img_btn_down, NULL, screen, &dest);
        }
        else
        {
          SDL_BlitSurface(img_btn_up, NULL, screen, &dest);
        }
	
        dest.x = 640 - 96 + ((i % 2) * 48) + 4;
        dest.y = ((i / 2) * 48) + 40 + 4;

        SDL_BlitSurface(img_magics[i], NULL, screen, &dest);

	
        dest.x = 640 - 96 + ((i % 2) * 48) + 4 +
		 (40 - img_magic_names[i]->w) / 2;
        dest.y = ((i / 2) * 48) + 40 + 4 + (44 - img_magic_names[i]->h);

        SDL_BlitSurface(img_magic_names[i], NULL, screen, &dest);
      }
      else
      {
        SDL_BlitSurface(img_btn_off, NULL, screen, &dest);	
      }
    }
}


/* Draw color selector: */

void draw_colors(int enabled)
{
  int i;
  SDL_Rect dest;

  dest.x = 0;
  dest.y = 40 + ((NUM_TOOLS / 2) * 48);

  if (enabled)
    SDL_BlitSurface(img_titles[TITLE_COLORS], NULL, screen, &dest);
  else
    SDL_BlitSurface(img_titles[TITLE_NOCOLORS], NULL, screen, &dest);

  for (i = 0; i < NUM_COLORS; i++)
  {
#ifndef LOW_QUALITY_COLOR_SELECTOR
    dest.x = (i * ((640 - 96) / NUM_COLORS)) + 96;
    dest.y = 40 + ((NUM_TOOLS / 2) * 48);

    if (enabled)
      SDL_BlitSurface(img_color_btns[i], NULL, screen, &dest);
    else
      SDL_BlitSurface(img_color_btns[COLOR_WHITE], NULL, screen, &dest);
#else
    dest.x = (i * ((640 - 96) / NUM_COLORS)) + 96;
    dest.y = 40 + ((NUM_TOOLS / 2) * 48);
    dest.w = ((640 - 96) / NUM_COLORS);
    dest.h = 48;

    if (enabled)
      SDL_FillRect(screen, &dest,
		   SDL_MapRGB(screen->format,
			      color_hexes[i][0],
			      color_hexes[i][1],
			      color_hexes[i][2]));
    else
      SDL_FillRect(screen, &dest,
		   SDL_MapRGB(screen->format, 240, 240, 240));
#endif

    if (i == cur_color && enabled)
    {
      dest.x = (i * ((640 - 96) / NUM_COLORS)) + 96;
      dest.y = 40 + ((NUM_TOOLS / 2) * 48);

      SDL_BlitSurface(img_paintcan, NULL, screen, &dest);
    }
  }
}


/* Draw brushes: */

void draw_brushes(void)
{
  int i, off_y, max, brush;
  SDL_Rect dest;


  /* Draw the title: */

  dest.x = 640 - 96;
  dest.y = 0;
  SDL_BlitSurface(img_titles[TITLE_BRUSHES], NULL, screen, &dest);


  /* Do we need scrollbars? */

  if (num_brushes > 14)
  {
    off_y = 24;
    max = 12;

    dest.x = 640 - 96;
    dest.y = 40;

    if (brush_scroll > 0)
    {
      SDL_BlitSurface(img_scroll_up, NULL, screen, &dest);
    }
    else
    {
      SDL_BlitSurface(img_scroll_up_off, NULL, screen, &dest);
    }

    dest.x = 640 - 96;
    dest.y = 40 + 24 + (6 * 48);

    if (brush_scroll < num_brushes - 12)
    {
      SDL_BlitSurface(img_scroll_down, NULL, screen, &dest);
    }
    else
    {
      SDL_BlitSurface(img_scroll_down_off, NULL, screen, &dest);
    }
  }
  else
  {
    off_y = 0;
    max = 14;
  }


  /* Draw each of the shown brushes: */

  for (brush = brush_scroll;
       brush < brush_scroll + max;
       brush++)
  {
    i = brush - brush_scroll;


    dest.x = ((i % 2) * 48) + (640 - 96);
    dest.y = ((i / 2) * 48) + 40 + off_y;

    if (brush == cur_brush)
    {
      SDL_BlitSurface(img_btn_down, NULL, screen, &dest);
    }
    else if (brush < num_brushes)
    {
      SDL_BlitSurface(img_btn_up, NULL, screen, &dest);
    }
    else
    {
      SDL_BlitSurface(img_btn_off, NULL, screen, &dest);
    }

    if (brush < num_brushes)
    {
      dest.x = ((i % 2) * 48) + (640 - 96) +
	       ((48 - (img_brushes[brush]->w)) / 2);

      dest.y = ((i / 2) * 48) + 40 + ((48 - (img_brushes[brush]->w)) / 2) +
	        off_y;

      SDL_BlitSurface(img_brushes[brush], NULL, screen, &dest);
    }
  }
}


/* Draw stamps: */

void draw_stamps(void)
{
  int i, off_y, max, stamp;
  SDL_Rect dest;
  SDL_Surface * img;


  /* Draw the title: */

  dest.x = 640 - 96;
  dest.y = 0;
  SDL_BlitSurface(img_titles[TITLE_STAMPS], NULL, screen, &dest);


  /* Do we need scrollbars? */

  if (num_stamps > 14)
  {
    off_y = 24;
    max = 12;

    dest.x = 640 - 96;
    dest.y = 40;

    if (stamp_scroll > 0)
    {
      SDL_BlitSurface(img_scroll_up, NULL, screen, &dest);
    }
    else
    {
      SDL_BlitSurface(img_scroll_up_off, NULL, screen, &dest);
    }

    dest.x = 640 - 96;
    dest.y = 40 + 24 + (6 * 48);

    if (stamp_scroll < num_stamps - 12)
    {
      SDL_BlitSurface(img_scroll_down, NULL, screen, &dest);
    }
    else
    {
      SDL_BlitSurface(img_scroll_down_off, NULL, screen, &dest);
    }
  }
  else
  {
    off_y = 0;
    max = 14;
  }


  /* Draw each of the shown stamps: */

  for (stamp = stamp_scroll;
       stamp < stamp_scroll + max;
       stamp++)
  {
    i = stamp - stamp_scroll;


    dest.x = ((i % 2) * 48) + (640 - 96);
    dest.y = ((i / 2) * 48) + 40 + off_y;

    if (stamp == cur_stamp)
    {
      SDL_BlitSurface(img_btn_down, NULL, screen, &dest);
    }
    else if (stamp < num_stamps)
    {
      SDL_BlitSurface(img_btn_up, NULL, screen, &dest);
    }
    else
    {
      SDL_BlitSurface(img_btn_off, NULL, screen, &dest);
    }

    if (stamp < num_stamps)
    {
      if (img_stamp_thumbs[stamp] != NULL)
	img = img_stamp_thumbs[stamp];
      else
	img = img_stamps[stamp];

      dest.x = ((i % 2) * 48) + (640 - 96) +
	       ((48 - (img->w)) / 2);

      dest.y = ((i / 2) * 48) + 40 + ((48 - (img->h)) / 2) + off_y;

      SDL_BlitSurface(img, NULL, screen, &dest);
    }
  }
}


/* Draw the shape selector: */

void draw_shapes(void)
{
  int i;
  SDL_Rect dest;


  dest.x = 640 - 96;
  dest.y = 0;
  SDL_BlitSurface(img_titles[TITLE_SHAPES], NULL, screen, &dest);

  for (i = 0; i < 14; i++)
    {
      dest.x = ((i % 2) * 48) + 640 - 96;
      dest.y = ((i / 2) * 48) + 40;


      if (i == cur_shape)
	{
	  SDL_BlitSurface(img_btn_down, NULL, screen, &dest);
	}
      else if (i < NUM_SHAPES)
	{
	  SDL_BlitSurface(img_btn_up, NULL, screen, &dest);
	}
      else
	{
	  SDL_BlitSurface(img_btn_off, NULL, screen, &dest);
	}


      if (i < NUM_SHAPES)
	{
	  dest.x = ((i % 2) * 48) + 4 + 640 - 96;
	  dest.y = ((i / 2) * 48) + 40 + 4;
	
	  SDL_BlitSurface(img_shapes[i], NULL, screen, &dest);

	  dest.x = ((i % 2) * 48) + 4 + 640 - 96 +
		   (40 - img_shape_names[i]->w) / 2;
          dest.y = ((i / 2) * 48) + 40 + 4 + (44 - img_shape_names[i]->h);

          SDL_BlitSurface(img_shape_names[i], NULL, screen, &dest);
	}
    }
}


/* Draw no selectables: */

void draw_none(void)
{
  int i;
  SDL_Rect dest;

  dest.x = 640 - 96;
  dest.y = 0;
  SDL_BlitSurface(img_titles[TITLE_NONE], NULL, screen, &dest);

  for (i = 0; i < 14; i++)
    {
      dest.x = ((i % 2) * 48) + 640 - 96;
      dest.y = ((i / 2) * 48) + 40;

      SDL_BlitSurface(img_btn_off, NULL, screen, &dest);
    }
}


/* Load an arbitrary set of images into an array (e.g., brushes or stamps) */

#ifndef NOSOUND
void loadarbitrary(SDL_Surface * surfs[], char * descs[], info_type * infs[],
		   Mix_Chunk * sounds[],
		   int * count, int starting, int max,
		   char * dir)
#else
void loadarbitrary(SDL_Surface * surfs[], char * descs[], info_type * infs[],
		   int * count, int starting, int max,
		   char * dir)
#endif
{
  DIR * d;
  struct dirent * f;
  struct stat sbuf;
  char fname[512];
  char * d_names[MAX_FILES];
  int num_files, i;


  *count = starting;

  /* Open the directory: */

  d = opendir(dir);
  if (d == NULL)
  {
    fprintf(stderr,
	    "\nError: I can't find a directory of images\n"
	    "%s\n"
	    "The system error that occured was: %s\n",
	    dir, strerror(errno));

    cleanup();
    exit(1);
  }


  /* Read directory for images: */

  num_files = 0;
  do
  {
    f = readdir(d);

    if (f != NULL)
    {
      d_names[num_files] = strdup(f->d_name);
      num_files++;
    }
  }
  while (f != NULL && num_files < MAX_FILES);

  closedir(d);


  qsort(d_names, num_files, sizeof(char *),
	(int(*)(const void *, const void *))compare_strings);


  /* Do something with each file (load if PNG, recurse if directory): */

  for (i = 0; i < num_files && *count < max; i++)
    {
      /* Ignore things starting with "." (e.g., "." and ".." dirs): */

      if (strstr(d_names[i], ".") != d_names[i])
      {
        /* If it's a directory, recurse down into it: */

  	snprintf(fname, sizeof(fname), "%s/%s", dir, d_names[i]);
	debug(fname);
	
	stat(fname, &sbuf);
	
	if (S_ISDIR(sbuf.st_mode))
	{
          debug("...is a directory");

#ifndef NOSOUND
	  loadarbitrary(surfs, descs, infs, sounds, count, *count, max, fname);
#else
	  loadarbitrary(surfs, descs, infs, count, *count, max, fname);
#endif
	}
        else if (strstr(d_names[i], ".png") != NULL)
        {
          /* If it has ".png" in the filename, assume we can try to load it: */
	
	  surfs[*count] = loadimage(fname);


	  /* Check for a companion ".txt" file! */

	  if (descs != NULL)
	    descs[*count] = loaddesc(fname);

	  if (infs != NULL)
	    infs[*count] = loadinfo(fname);


#ifndef NOSOUND
	  if (use_sound)
	  {
	    if (sounds != NULL)
	      sounds[*count] = loadsound(fname);
	  }
#endif


	  *count = *count + 1;

          show_progress_bar();
        }
      }

      free(d_names[i]);
    }


  /* Give warning if too many files were found (e.g., some not loaded): */

  if (*count == max)
  {
    fprintf(stderr,
	    "\nWarning: Reached maximum images (%d) which can be stored in:\n"
	    "%s\n\n",
	    max, dir);
  }

  debug("loadarbitrary() ends...");
}


/* Create a thumbnail: */

SDL_Surface * thumbnail(SDL_Surface * src, int max_x, int max_y,
			int keep_aspect)
{
  int x, y, src_x, src_y, off_x, off_y;
  SDL_Surface * s;
  Uint32 amask, tr, tg, tb, ta;
  Uint8 r, g, b, a;
  float xscale, yscale;
  int tmp;


  /* Determine scale and centering offsets: */

  if (!keep_aspect)
    {
      yscale = (float) ((float) src->h / (float) max_y);
      xscale = (float) ((float) src->w / (float) max_x);

      // off_x = ((src->h / yscale) - (src->w / xscale)) / 2;
      // off_y = ((src->w / xscale) - (src->h / yscale)) / 2;

      off_x = 0;
      off_y = 0;
    }
  else
    {
      if (src->h > src->w)
	{
	  yscale = (float) ((float) src->h / (float) max_y);
	  xscale = yscale;
	  off_x = ((src->h - src->w) / xscale) / 2;
	  off_y = 0;
	}
      else
	{
	  xscale = (float) ((float) src->w / (float) max_x);
	  yscale = xscale;
	  off_x = 0;
	  off_y = ((src->w - src->h) / xscale) / 2;
	}
    }


  /* Create surface for thumbnail: */

  if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
    amask = 0x000000FF;
  else
    amask = 0xFF000000;

  s = SDL_CreateRGBSurface(SDL_SWSURFACE,
		           max_x, max_y,
			   src->format->BitsPerPixel,
			   src->format->Rmask,
			   src->format->Gmask,
			   src->format->Bmask,
			   amask);


  if (s == NULL)
  {
    fprintf(stderr, "\nError: Can't build stamp thumbnails\n"
		    "The Simple DirectMedia Layer error that occured was:\n"
		    "%s\n\n", SDL_GetError());

    cleanup();
    exit(1);
  }


  /* Create thumbnail itself: */

  SDL_LockSurface(src);
  SDL_LockSurface(s);

  for (y = 0; y < max_y; y++)
  {
    for (x = 0; x < max_x; x++)
    {
#ifndef LOW_QUALITY_THUMBNAILS
      tr = 0;
      tg = 0;
      tb = 0;
      ta = 0;

      tmp = 0;

      for (src_y = y * yscale; src_y < y * yscale + yscale &&
		               src_y < src->h; src_y++)
      {
        for (src_x = x * xscale; src_x < x * xscale + xscale &&
			         src_x < src->w; src_x++)
	{
	  SDL_GetRGBA(getpixel(src, src_x, src_y),
		      src->format,
		      &r, &g, &b, &a);

	  tr = tr + r;
	  tb = tb + b;
	  tg = tg + g;
	  ta = ta + a;

	  tmp++;
	}
      }

      if (tmp != 0)
      {
        tr = tr / tmp;
        tb = tb / tmp;
        tg = tg / tmp;
        ta = ta / tmp;

        putpixel(s, x + off_x, y + off_y, SDL_MapRGBA(s->format,
			                              (Uint8) tr,
				                      (Uint8) tg,
				                      (Uint8) tb,
				                      (Uint8) ta));
      }
#else
      src_x = x * xscale;
      src_y = y * yscale;

      putpixel(s, x + off_x, y + off_y, getpixel(src, src_x, src_y));
#endif
    }
  }

  SDL_UnlockSurface(s);
  SDL_UnlockSurface(src);

  return s;
}


/* Get a pixel: */

Uint32 getpixel(SDL_Surface * surface, int x, int y)
{
  int bpp;
  Uint8 * p;
  Uint32 pixel;

  pixel = 0;


  /* Assuming the X/Y values are within the bounds of this surface... */

  if (x >= 0 && y >= 0 && x < surface -> w && y < surface -> h)
    {
      /* SDL_LockSurface(surface); */


      /* Determine bytes-per-pixel for the surface in question: */

      bpp = surface->format->BytesPerPixel;


      /* Set a pointer to the exact location in memory of the pixel
         in question: */

      p = (Uint8 *) (((Uint8 *)surface->pixels) +  /* Start at top of RAM */
	 	      (y * surface->pitch) +  /* Go down Y lines */
	      	      (x * bpp));             /* Go in X pixels */


      /* Return the correctly-sized piece of data containing the
       * pixel's value (an 8-bit palette value, or a 16-, 24- or 32-bit
       * RGB value) */

      if (bpp == 1)         /* 8-bit display */
        pixel = *p;
      else if (bpp == 2)    /* 16-bit display */
        pixel = *(Uint16 *)p;
      else if (bpp == 3)    /* 24-bit display */
        {
          /* Depending on the byte-order, it could be stored RGB or BGR! */

          if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
            pixel = p[0] << 16 | p[1] << 8 | p[2];
          else
            pixel = p[0] | p[1] << 8 | p[2] << 16;
        }
      else if (bpp == 4)    /* 32-bit display */
        pixel = *(Uint32 *)p;

      /* SDL_UnlockSurface(surface); */
    }

  return pixel;
}


/* Draw a single pixel into the surface: */

void putpixel(SDL_Surface * surface, int x, int y, Uint32 pixel)
{
  int bpp;
  Uint8 * p;


  /* Assuming the X/Y values are within the bounds of this surface... */

  if (x >= 0 && y >= 0 && x < surface->w && y < surface->h)
    {
      /* SDL_LockSurface(surface); */


      /* Determine bytes-per-pixel for the surface in question: */

      bpp = surface->format->BytesPerPixel;


      /* Set a pointer to the exact location in memory of the pixel
       *          in question: */

      p = (Uint8 *) (((Uint8 *)surface->pixels) + /* Start: beginning of RAM */
		     (y * surface->pitch) +  /* Go down Y lines */
                     (x * bpp));             /* Go in X pixels */


      /* Set the (correctly-sized) piece of data in the surface's RAM
       *          to the pixel value sent in: */

      if (bpp == 1)
        *p = pixel;
      else if (bpp == 2)
        *(Uint16 *)p = pixel;
      else if (bpp == 3)
        {
          if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
            {
              p[0] = (pixel >> 16) & 0xff;
              p[1] = (pixel >> 8) & 0xff;
              p[2] = pixel & 0xff;
            }
          else
            {
              p[0] = pixel & 0xff;
              p[1] = (pixel >> 8) & 0xff;
              p[2] = (pixel >> 16) & 0xff;
            }
        }
      else if (bpp == 4)
        {
          *(Uint32 *)p = pixel;
        }

      /* SDL_UnlockSurface(surface); */
    }
}


/* Show debugging stuff: */

void debug(char * str)
{
#ifdef DEBUG
  fprintf(stderr, "DEBUG: %s\n", str);
  fflush(stderr);
#endif
}


/* Undo! */

void do_undo(void)
{
  int wanna_update_toolbar;

  wanna_update_toolbar = 0;

  if (cur_undo != oldest_undo)
    {
      cur_undo--;

      if (cur_undo < 0)
	cur_undo = NUM_UNDO_BUFS - 1;

#ifdef DEBUG
      printf("BLITTING: %d\n", cur_undo);
#endif
      SDL_BlitSurface(undo_bufs[cur_undo], NULL, canvas, NULL);
      update_canvas(0, 0, (640 - 96), (48 * 7) + 40);

      if (cur_undo == oldest_undo)
	{
	  tool_avail[TOOL_UNDO] = 0;
	  wanna_update_toolbar = 1;
	}

      if (tool_avail[TOOL_REDO] == 0)
	{
	  tool_avail[TOOL_REDO] = 1;
	  wanna_update_toolbar = 1;
	}

      if (wanna_update_toolbar)
	{
	  draw_toolbar();
	  SDL_UpdateRect(screen, 0, 0, 96, (48 * 7) + 40);
	}

      been_saved = 0;
    }

#ifdef DEBUG
  printf("UNDO: Current=%d  Oldest=%d  Newest=%d\n",
	 cur_undo, oldest_undo, newest_undo);
#endif
}


/* Redo! */

void do_redo(void)
{
  if (cur_undo != newest_undo)
    {
      cur_undo = (cur_undo + 1) % NUM_UNDO_BUFS;

#ifdef DEBUG
      printf("BLITTING: %d\n", cur_undo);
#endif
      SDL_BlitSurface(undo_bufs[cur_undo], NULL, canvas, NULL);
      update_canvas(0, 0, (640 - 96), (48 * 7) + 40);

      been_saved = 0;
    }

#ifdef DEBUG
  printf("REDO: Current=%d  Oldest=%d  Newest=%d\n",
	 cur_undo, oldest_undo, newest_undo);
#endif


  if (((cur_undo + 1) % NUM_UNDO_BUFS) == newest_undo)
    {
      tool_avail[TOOL_REDO] = 0;
    }

  tool_avail[TOOL_UNDO] = 1;

  draw_toolbar();
  SDL_UpdateRect(screen, 0, 0, 96, (48 * 7) + 40);
}


/* Create the current brush in the current color: */

void render_brush(void)
{
  Uint32 amask;
  int x, y;
  Uint8 r, g, b, a;


  /* Free the old rendered brush (if any): */

  if (img_cur_brush != NULL)
    {
      SDL_FreeSurface(img_cur_brush);
    }


  /* Create a surface to render into: */

  /* FIXME: Is this right?  Seems there are issues on Sparcs? */

  /*
  if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
    amask = 0x000000FF;
  else
    amask = 0xFF000000;
  */

  amask = ~(img_brushes[cur_brush]->format->Rmask |
	    img_brushes[cur_brush]->format->Gmask |
	    img_brushes[cur_brush]->format->Bmask);

  img_cur_brush =
    SDL_CreateRGBSurface(SDL_SWSURFACE,
			 img_brushes[cur_brush]->w,
			 img_brushes[cur_brush]->h,
			 img_brushes[cur_brush]->format->BitsPerPixel,
			 img_brushes[cur_brush]->format->Rmask,
			 img_brushes[cur_brush]->format->Gmask,
			 img_brushes[cur_brush]->format->Bmask,
			 amask);

  if (img_cur_brush == NULL)
  {
    fprintf(stderr, "\nError: Can't render a brush!\n"
		    "The Simple DirectMedia Layer error that occured was:\n"
		    "%s\n\n", SDL_GetError());

    cleanup();
    exit(1);
  }


  /* Render the new brush: */

  SDL_LockSurface(img_brushes[cur_brush]);
  SDL_LockSurface(img_cur_brush);

  for (y = 0; y < img_brushes[cur_brush]->h; y++)
    {
      for (x = 0; x < img_brushes[cur_brush]->w; x++)
	{
	  SDL_GetRGBA(getpixel(img_brushes[cur_brush], x, y),
		     img_brushes[cur_brush]->format,
		     &r, &g, &b, &a);
	
	  putpixel(img_cur_brush, x, y,
		   SDL_MapRGBA(img_cur_brush->format,
			       color_hexes[cur_color][0],
			       color_hexes[cur_color][1],
			       color_hexes[cur_color][2],
			       a));
	}
    }

  SDL_UnlockSurface(img_cur_brush);
  SDL_UnlockSurface(img_brushes[cur_brush]);

  brush_counter = 0;
}


/* Play a sound: */

void playsound(int chan, int s, int override)
{
#ifndef NOSOUND
  if (use_sound)
  {
    if (override || !Mix_Playing(chan))
      Mix_PlayChannel(chan, sounds[s], 0);
  }
#endif
}


/* Draw a XOR line: */

void line_xor(int x1, int y1, int x2, int y2)
{
  int dx, dy, y;
  float m, b;


  SDL_LockSurface(screen);

  dx = x2 - x1;
  dy = y2 - y1;

  if (dx != 0)
    {
      m = ((float) dy) / ((float) dx);
      b = y1 - m * x1;

      if (x2 >= x1)
	dx = 1;
      else
	dx = -1;


      while (x1 != x2)
	{
	  y1 = m * x1 + b;
	  y2 = m * (x1 + dx) + b;

	  if (y1 > y2)
	    {
	      y = y1;
	      y1 = y2;
	      y2 = y;
	    }
	
	  for (y = y1; y <= y2; y++)
	    {
	      clipped_putpixel(screen, x1 + 96, y,
		               0xFFFFFFFF - getpixel(screen, x1 + 96, y));
	    }
	
	  x1 = x1 + dx;
	}
    }
  else
    {
      if (y1 > y2)
	{
	  y = y1;
	  y1 = y2;
	  y2 = y;
	}

      for (y = y1; y <= y2; y++)
        {
	  clipped_putpixel(screen, x1 + 96, y,
		          0xFFFFFFFF - getpixel(screen, x1 + 96, y));
        }
    }

  SDL_UnlockSurface(screen);
}


/* Should really clip at the line level, but oh well... */

void clipped_putpixel(SDL_Surface * dest, int x, int y, Uint32 c)
{
  if (x >= 96 && x < (640 - 96) &&
      y >= 0 && y < (48 * 7 + 40))
  {
    putpixel(dest, x, y, c);
  }
}


/* Draw a XOR rectangle: */

void rect_xor(int x1, int y1, int x2, int y2)
{
  if (x1 < 0)
    x1 = 0;

  if (x2 < 0)
    x2 = 0;

  if (y1 < 0)
    y1 = 0;

  if (y2 < 0)
    y2 = 0;

  if (x1 >= (640 - 96 - 96))
    x1 = (640 - 96 - 96) - 1;

  if (x2 >= (640 - 96 - 96))
    x2 = (640 - 96 - 96) - 1;

  if (y1 >= (48 * 7) + 40)
    y1 = (48 * 7) + 40 - 1;

  if (y2 >= (48 * 7) + 40)
    y2 = (48 * 7) + 40 - 1;

  line_xor(x1, y1, x2, y1);
  line_xor(x2, y1, x2, y2);
  line_xor(x1, y2, x2, y2);
  line_xor(x1, y1, x1, y2);
}


/* Erase at the cursor! */

void do_eraser(int x, int y)
{
  SDL_Rect dest;

  dest.x = x - 48;
  dest.y = y - 48;
  dest.w = 96;
  dest.h = 96;

  SDL_FillRect(canvas, &dest,
	       SDL_MapRGB(canvas->format, 255, 255, 255));


#ifndef NOSOUND
  if (use_sound)
    {
      if (!Mix_Playing(0))
	{
	  eraser_sound = (eraser_sound + 1) % 2;
	
	  playsound(0, SND_ERASER1 + eraser_sound, 0);
	}
    }
#endif

  update_canvas(x - 48, y - 48, x + 48, y + 48);

  rect_xor(x - 48, y - 48,
           x + 48, y + 48);
}


/* Reset available tools (for new image / starting out): */

void reset_avail_tools(void)
{
  int i;

  for (i = 0; i < NUM_TOOLS; i++)
    {
      tool_avail[i] = 1;
    }


  /* Unavailable at the beginning of a new canvas: */

  tool_avail[TOOL_UNDO] = 0;
  tool_avail[TOOL_REDO] = 0;
  tool_avail[TOOL_NEW] = 0;

  if (been_saved)
    tool_avail[TOOL_SAVE] = 0;


  /* Unavailable in rare circumstances: */

  if (num_stamps == 0)
    tool_avail[TOOL_STAMP] = 0;


  /* Disable quit? */

  if (disable_quit)
    tool_avail[TOOL_QUIT] = 0;


  /* Unavailable because they don't work or are broken: */

#ifdef HIDE_UNIMPLEMENTED
  tool_avail[TOOL_TEXT] = 0;
  tool_avail[TOOL_PRINT] = 0;
#endif
}


/* Save and disable available tools (for Open-Dialog) */

void disable_avail_tools(void)
{
  int i;

  for (i = 0; i < NUM_TOOLS; i++)
  {
    tool_avail_bak[i] = tool_avail[i];
    tool_avail[i]=0;
  }
}

/* Restore and enable available tools (for End-Of-Open-Dialog) */

void enable_avail_tools(void)
{
  int i;

  for (i = 0; i < NUM_TOOLS; i++)
  {
    tool_avail[i] = tool_avail_bak[i];
  }
}


/* Update a rect. based on two x/y coords (not necessarly in order): */

void update_screen(int x1, int y1, int x2, int y2)
{
  int tmp;

  if (x1 > x2)
  {
    tmp = x1;
    x1 = x2;
    x2 = tmp;
  }

  if (y1 > y2)
  {
    tmp = y1;
    y1 = y2;
    y2 = tmp;
  }

  x1 = x1 - 1;
  x2 = x2 + 1;
  y1 = y1 - 1;
  y2 = y2 + 1;


  if (x1 < 0)
    x1 = 0;
  if (x2 < 0)
    x2 = 0;
  if (y1 < 0)
    y1 = 0;
  if (y2 < 0)
    y2 = 0;

  if (x1 >= 640)
    x1 = 640 - 1;
  if (x2 >= 640)
    x2 = 640 - 1;
  if (y1 >= 480)
    y1 = 480 - 1;
  if (y2 >= 480)
    y2 = 480 - 1;

  SDL_UpdateRect(screen, x1, y1, x2 - x1 + 1, y2 - y1 + 1);
}


/* Build a color based on two colors and an alpha... */

Uint8 alpha(Uint8 c1, Uint8 c2, Uint8 a)
{
  Uint16 c, nc1, nc2, na;

  na = a;
  nc1 = c1;
  nc2 = c2;

  if (nc1 > 200)
    nc1 = 200;

  c = ((nc1 * na) / 255 + (nc2 * (255 - na)) / 255);

  return (Uint8) c;
}


/* For qsort() call in loadarbitrary()... */

int compare_strings(char * * s1, char * * s2)
{
  return (strcmp(*s1, *s2));
}


/* Draw tux's text on the screen: */

void draw_tux_text(int which_tux, char * str)
{
  SDL_Rect dest;
  SDL_Color black = {0, 0, 0, 0};


  /* Clear first: */

  dest.x = 0;
  dest.y = (48 * 7) + 40 + 48;
  dest.w = 640;
  dest.h = 480 - ((48 * 7) + 40 + 48);

  SDL_FillRect(screen, &dest, SDL_MapRGB(screen->format, 255, 255, 255));


  /* Draw tux: */

  dest.x = 0;
  dest.y = 480 - (img_tux[which_tux] -> h);

  if (dest.y < ((48 * 7) + 40 + 48))
    dest.y = ((48 * 7) + 40 + 48);

  SDL_BlitSurface(img_tux[which_tux], NULL, screen, &dest);


  wordwrap_text(font, str, black,
		img_tux[which_tux] -> w + 5, (48 * 7) + 40 + 48 + 5,
	       	640);


  /* Update the display: */

  SDL_UpdateRect(screen,
		 0, (48 * 7) + 40 + 48,
		 640, 480 - ((48 * 7) + 40 + 48));
}


void wordwrap_text(TTF_Font * font, char * str, SDL_Color color,
		   int left, int top, int right)
{
  int x, y, i, j;
  char substr[512];
  int len;
  SDL_Surface * text;
  SDL_Rect dest;


  /* Cursor begins at the right side of tux: */

  x = left;
  y = top;

  if (strlen(str) > sizeof(substr) - 1)
    str[sizeof(substr) - 1] = '\0';


  /* For each word... */

  for (i = 0; i < strlen(str); i++)
  {
    /* Figure out the word... */

    len = 0;

    for (j = i; str[j] != ' ' && str[j] != '\0'; j++)
    {
      substr[len++] = str[j];
    }

    substr[len++] = ' ';
    substr[len] = '\0';


    /* Render the word for display... */

    text = RENDER_TEXT(font, substr, color);


    /* If it won't fit on this line, move to the next! */

    if (x + text->w > right)  /* Correct? */
    {
      x = left;
      y = y + text->h;
    }


    /* Draw the word: */

    dest.x = x;
    dest.y = y;

    SDL_BlitSurface(text, NULL, screen, &dest);


    /* Move the cursor one word's worth: */

    x = x + text->w;


    /* Free the temp. surface: */

    SDL_FreeSurface(text);


    /* Now on to the next word... */

    i = j;
  }
}


/* Load a file's sound: */

#ifndef NOSOUND

Mix_Chunk * loadsound(char * fname)
{
  char * snd_fname;
  char tmp_str[64];
  Mix_Chunk * tmp_snd;


  debug(fname);


  /* First, check for localized version of sound: */

  snd_fname = malloc(strlen(fname) + strlen(lang_prefixes[language]) + 2);

  strcpy(snd_fname, fname);
  snprintf(tmp_str, strlen(tmp_str), "_%s.wav", lang_prefixes[language]);


  if (strstr(snd_fname, ".png") != NULL)
  {
    strcpy(strstr(snd_fname, ".png"), tmp_str);
    debug(snd_fname);

    tmp_snd = Mix_LoadWAV(snd_fname);

    if (tmp_snd == NULL)
    {
      debug("...No local version of sound!");

      /* Now, check for default sound: */

      free(snd_fname);

      snd_fname = strdup(fname);

      if (strstr(snd_fname, ".png") != NULL)
      {
        strcpy(strstr(snd_fname, ".png"), ".wav");
        debug(snd_fname);
        tmp_snd = Mix_LoadWAV(snd_fname);
        free(snd_fname);

        if (tmp_snd == NULL)
	{
	  debug("...No default version of sound!");
          return NULL;
	}

        return (tmp_snd);
      }
      else
      {
        return NULL;
      }
    }

    return (tmp_snd);
  }
  else
  {
    return NULL;
  }
}

#endif


/* Load a file's description: */

char * loaddesc(char * fname)
{
  char * txt_fname;
  char buf[256], def_buf[256];
  int found, got_first;
  FILE * fi;


  txt_fname = strdup(fname);

  if (strstr(txt_fname, ".png") != NULL)
  {
    strcpy(strstr(txt_fname, ".png"), ".txt");

    fi = fopen(txt_fname, "r");

    if (fi == NULL)
    {
      /*
        fprintf(stderr, "\nWarning: Couldn't open a description file:\n");
        perror(txt_fname);
        fprintf(stderr, "\n");
      */

      free(txt_fname);

      return NULL;
    }

    free(txt_fname);

    got_first = 0;
    found = 0;

    strcpy(def_buf, "");

    do
    {
      fgets(buf, sizeof(buf), fi);

      if (!feof(fi))
      {
        /* Strip any trailing spaces: */

        while (buf[strlen(buf) - 1] == ' ' ||
	       buf[strlen(buf) - 1] == '\r' ||
	       buf[strlen(buf) - 1] == '\n')
        {
          buf[strlen(buf) - 1] = '\0';
        }


        if (!got_first)
        {
          /* First one is the default: */

          strcpy(def_buf, buf);
	  got_first = 1;
        }


        /* See if it's the one for this locale... */
	
        if (strstr(buf, lang_prefixes[language]) == buf)
        {
          if (buf[strlen(lang_prefixes[language])] == '=')
	    found = 1;
	}
      }
    }
    while (!feof(fi) && !found);

    fclose(fi);


    /* Return the string: */

    if (found)
      return(strdup(buf + (strlen(lang_prefixes[language])) + 1));
    else
      return(strdup(def_buf));
  }
  else
  {
    return NULL;
  }
}


/* Load a file's info: */

info_type * loadinfo(char * fname)
{
  char * dat_fname;
  char buf[256];
  info_type inf;
  info_type * inf_ret;
  FILE * fi;


  /* Clear info struct first: */

  inf.colorable = 0;
  inf.tintable = 0;


  /* Load info! */

  dat_fname = strdup(fname);

  if (strstr(dat_fname, ".png") != NULL)
  {
    strcpy(strstr(dat_fname, ".png"), ".dat");

    fi = fopen(dat_fname, "r");

    if (fi == NULL)
    {
      /*
        fprintf(stderr, "\nWarning: Couldn't open an info file:\n");
        perror(txt_fname);
        fprintf(stderr, "\n");
      */

      free(dat_fname);

      return NULL;
    }

    free(dat_fname);


    do
    {
      fgets(buf, sizeof(buf), fi);

      if (!feof(fi))
      {
        /* Strip any trailing spaces: */

        while (buf[strlen(buf) - 1] == ' ' ||
	       buf[strlen(buf) - 1] == '\r' ||
	       buf[strlen(buf) - 1] == '\n')
        {
          buf[strlen(buf) - 1] = '\0';
        }

	if (strcmp(buf, "colorable") == 0)
	  inf.colorable = 1;
	else if (strcmp(buf, "tintable") == 0)
	  inf.tintable = 1;
      }
    }
    while (!feof(fi));

    fclose(fi);


    /* Return the info: */
   
    inf_ret = malloc(sizeof(info_type));
    /* FIXME: Check for errors! */

    memcpy(inf_ret, &inf, sizeof(info_type));

    return(inf_ret);
  }
  else
  {
    return NULL;
  }
}


/* Wait for a keypress or mouse click */

void do_wait(void)
{
  SDL_Event event;
  int done, counter;

  done = 0;

  counter = 50;  /* About 5 seconds */

  do
  {
    while (SDL_PollEvent(&event))
    {
      if (event.type == SDL_QUIT)
      {
        done = 1;

        /* FIXME: Handle SDL_Quit better */
      }
      else if (event.type == SDL_KEYDOWN)
      {
        done = 1;
      }
      else if (event.type == SDL_MOUSEBUTTONDOWN)
      {
        done = 1;
      }
    }

    counter--;
    SDL_Delay(100);
  }
  while (!done && counter > 0);
}


/* Load current (if any) image: */

void load_current(void)
{
  SDL_Surface * tmp;
  char * fname;
  char ftmp[1024];
  FILE * fi;


  /* Determine the current picture's ID: */

  fname = get_fname("current_id.txt");

  fi = fopen(fname, "r");
  if (fi == NULL)
  {
    fprintf(stderr,
            "\nWarning: Couldn't determine the current image's ID\n"
	    "%s\n"
	    "The system error that occured was:\n"
	    "%s\n\n", fname, strerror(errno));
    file_id[0] = '\0';
  }
  else
  {
    fgets(file_id, sizeof(file_id), fi);
    if (strlen(file_id) > 0)
    {
      file_id[strlen(file_id) - 1] = '\0';
    }
    fclose(fi);
  }

  free(fname);


  /* Load that image: */

  if (file_id[0] != '\0')
  {
    snprintf(ftmp, sizeof(ftmp), "saved/%s%s", file_id, FNAME_EXTENSION);

    fname = get_fname(ftmp);

#ifdef SAVE_AS_BMP
    tmp = SDL_LoadBMP(fname);
#else
    tmp = IMG_Load(fname);
#endif

    if (tmp == NULL)
    {
      fprintf(stderr,
              "\nWarning: Couldn't load any current image.\n"
	      "%s\n"
	      "The Simple DirectMedia Layer error that occured was:\n"
	      "%s\n\n", fname, SDL_GetError());

      file_id[0] = '\0';
    }
    else
    {
      SDL_BlitSurface(tmp, NULL, canvas, NULL);

      SDL_FreeSurface(tmp);

      tool_avail[TOOL_NEW] = 1;
    }

    free(fname);
  }
}


/* Save the current image to disk: */

void save_current(void)
{
  char * fname;
  int res;
  FILE * fi;


  fname = get_fname("");

  res = mkdir(fname, 0755);

  if (res != 0 && errno != EEXIST)
  {
    fprintf(stderr,
            "\nError: Can't create user data directory:\n"
	    "%s\n"
	    "The error that occured was:\n"
	    "%s\n\n", fname, strerror(errno));

    draw_tux_text(TUX_OOPS, strerror(errno));
  }

  free(fname);


  fname = get_fname("current_id.txt");

  fi = fopen(fname, "w");
  if (fi == NULL)
  {
    fprintf(stderr,
            "\nError: Can't keep track of current image.\n"
	    "%s\n"
	    "The error that occred was:\n"
	    "%s\n\n", fname, strerror(errno));

    draw_tux_text(TUX_OOPS, strerror(errno));
  }
  else
  {
    fprintf(fi, "%s\n", file_id);
    fclose(fi);
  }

  free(fname);
}


/* The filename for the current image: */

char * get_fname(char * name)
{
  char f[512];


#ifdef WIN32
  snprintf(f, sizeof(f), "userdata/%s", name);
#else
  if (getenv("HOME") != NULL)
  {
    snprintf(f, sizeof(f), "%s/.tuxpaint/%s",
	     getenv("HOME"), name);
  }
  else
  {
    strcpy(f, name);
  }
#endif

  return strdup(f);
}


/* Prompt the user with a yes/no question: */

int do_prompt(char * text, char * btn_yes, char * btn_no)
{
  SDL_Event event;
  SDL_Rect dest;
  int done, ans, w;
  SDL_Color black = {0, 0, 0, 0};
  SDLKey key;


  /* Draw button box: */

  playsound(0, SND_PROMPT, 1);

  for (w = 0; w <= 96; w = w + 4)
  {
    dest.x = 160 + 96 - w;
    dest.y = 94 + 96 - w;
    dest.w = (320 - 96 * 2) + w * 2;
    dest.h = w * 2;
    SDL_FillRect(screen, &dest, SDL_MapRGB(screen->format, 0, 0, 0));

    SDL_UpdateRect(screen, dest.x, dest.y, dest.w, dest.h);
    SDL_Delay(10);
  }

  w = w - 6;

  dest.x = 160 + 96 - w;
  dest.y = 94 + 96 - w;
  dest.w = (320 - 96 * 2) + w * 2;
  dest.h = w * 2;
  SDL_FillRect(screen, &dest, SDL_MapRGB(screen->format, 255, 255, 255));


  /* Draw the question: */

  wordwrap_text(font, text, black, 166, 100, 482);


  /* Draw yes button: */

  dest.x = 166;
  dest.y = 178;
  SDL_BlitSurface(img_yes, NULL, screen, &dest);

  wordwrap_text(font, btn_yes, black, 166 + 48 + 4, 183, 482);


  /* Draw no button: */

  if (strlen(btn_no) != 0)
  {
    dest.x = 166;
    dest.y = 230;
    SDL_BlitSurface(img_no, NULL, screen, &dest);

    wordwrap_text(font, btn_no, black, 166 + 48 + 4, 235, 482);
  }


  /* Draw Tux, waiting... */

  draw_tux_text(TUX_BORED, "");

  SDL_Flip(screen);

  done = 0;
  ans = 0;

  do
  {
    SDL_WaitEvent(&event);

    if (event.type == SDL_QUIT)
    {
      ans = 0;
      done = 1;
    }
    else if (event.type == SDL_KEYDOWN)
    {
      key = event.key.keysym.sym;

      /* FIXME: Should use different letters for different languages! */

      if (key == SDLK_y || key == SDLK_RETURN)
      {
	/* Y or ENTER - Yes! */
	
	ans = 1;
	done = 1;
      }
      else if (key == SDLK_n || key == SDLK_ESCAPE)
      {
	/* N or ESCAPE - No! */

	if (strlen(btn_no) != 0)
	{
	  ans = 0;
	  done = 1;
	}
	else
	{
	  if (key == SDLK_ESCAPE)
	  {
	    /* ESCAPE also simply dismisses if there's no Yes/No choice: */
	
	    ans = 1;
	    done = 1;
	  }
	}
      }
    }
    else if (event.type == SDL_MOUSEBUTTONDOWN)
    {
      if (event.button.x >= 166 && event.button.x < 166 + 48)
      {
	if (event.button.y >= 178 && event.button.y < 178 + 48)
	{
          ans = 1;
	  done = 1;
	}
	else if (strlen(btn_no) != 0 &&
		 event.button.y >= 230 && event.button.y < 230 + 48)
	{
	  ans = 0;
	  done = 1;
	}
      }
    }
  }
  while (!done);


  /* FIXME: Sound effect! */
  /* ... */


  /* Erase question prompt: */

  update_canvas(0, 0, 640 - 96 - 96, 48 * 7 + 40);

  return ans;
}


/* Free memory and prepare to quit: */

void cleanup(void)
{
  int i;

  for ( i = 0; i < MAX_STAMPS; ++i )
  {
    if ( txt_stamps[i] )
    {
      free(txt_stamps[i]);
      txt_stamps[i] = NULL;
    }
  }

  for ( i = 0; i < MAX_STAMPS; ++i )
  {
    if ( inf_stamps[i] )
    {
      free(inf_stamps[i]);
      inf_stamps[i] = NULL;
    }
  }

  free_surface_array( img_brushes, MAX_BRUSHES );
  free_surface_array( img_stamps, MAX_STAMPS );
  free_surface_array( img_tools, NUM_TOOLS );
  free_surface_array( img_tool_names, NUM_TOOLS );
  free_surface_array( img_titles, NUM_TITLES );
  free_surface_array( img_magics, NUM_MAGICS );
  free_surface_array( img_magic_names, NUM_MAGICS );
  free_surface_array( img_shapes, NUM_SHAPES );
  free_surface_array( img_shape_names, NUM_SHAPES );
  free_surface_array( img_tux, NUM_TIP_TUX );

  free_surface( &img_loading );
  free_surface( &img_progress );

  free_surface( &img_yes );
  free_surface( &img_no );

  free_surface( &img_open );
  free_surface( &img_erase );
  free_surface( &img_back );

  free_surface( &img_btn_up );
  free_surface( &img_btn_down );
  free_surface( &img_btn_off );

  free_surface( &img_cursor_up );
  free_surface( &img_cursor_down );

  free_surface( &img_scroll_up );
  free_surface( &img_scroll_down );
  free_surface( &img_scroll_up_off );
  free_surface( &img_scroll_down_off );

  free_surface( &img_paintcan );

  free_surface( &img_sparkles );

  free_surface_array( undo_bufs, NUM_UNDO_BUFS );
#ifndef LOW_QUALITY_COLOR_SELECTOR
  free_surface_array( img_color_btns, NUM_COLORS );
#endif
  free_surface_array( img_stamp_thumbs, MAX_STAMPS );

  free_surface( &screen );
  free_surface( &canvas );
  free_surface( &img_cur_brush );

  if (font != NULL)
  {
    TTF_CloseFont(font);
    font = NULL;
  }

  if (small_font != NULL)
  {
    TTF_CloseFont(small_font);
    small_font = NULL;
  }

#ifndef NOSOUND
  if (use_sound)
  {
    for (i = 0; i < NUM_SOUNDS; i++)
    {
      if (sounds[i])
      {
        Mix_FreeChunk(sounds[i]);
        sounds[i] = NULL;
      }
    }

    for (i = 0; i < num_stamps; i++)
    {
      if (snd_stamps[i])
      {
	Mix_FreeChunk(snd_stamps[i]);
	snd_stamps[i] = NULL;
      }
    }

    Mix_CloseAudio();
  }
#endif

  TTF_Quit();
  SDL_Quit();
}


void free_surface(SDL_Surface **surface_array)
{
  if (*surface_array)
  {
    SDL_FreeSurface(*surface_array);
    *surface_array = NULL;
  }
}


void free_surface_array(SDL_Surface *surface_array[], int count)
{
  int i;

  for (i = 0; i < count; ++i)
  {
    free_surface(&surface_array[i]);
  }
}


/* Draw a shape! */

void do_shape(int cx, int cy, int ox, int oy, int rotn, int use_brush)
{
  int side, angle_skip, init_ang, rx, ry, rmax, x1, y1, x2, y2, xp, yp,
    old_brush;
  float a1, a2, rotn_rad;
#ifdef SCAN_FILL
  point_type pts[1024];  /* Careful! */
  fpoint_type fpts_orig[1024], fpts_new[1024];
  int i;
  int num_pts;
#else
  int xx;
#endif


  /* Determine radius/shape of the shape to draw: */

  old_brush = 0;
  rx = ox - cx;
  ry = oy - cy;


  /* If the shape has a 1:1 ("locked") aspect ratio, use the larger radius: */

  if (shape_locked[cur_shape])
  {
    if (rx > ry)
      ry = rx;
    else
      rx = ry;
  }


  /* Render a default brush: */

  if (use_brush)
  {
    old_brush = cur_brush;
    cur_brush = 0;  /* Kludgy! */
    render_brush();
  }


  /* Draw the shape: */

  angle_skip = 360 / shape_sides[cur_shape];

  init_ang = shape_init_ang[cur_shape];


#ifdef SCAN_FILL
  num_pts = 0;
#endif

  for (side = 0; side < shape_sides[cur_shape]; side++)
  {
    a1 = (angle_skip * side + init_ang) * M_PI / 180;
    a2 = (angle_skip * (side + 1) + init_ang) * M_PI / 180;

    x1 = (int) (cos(a1) * rx);
    y1 = (int) (-sin(a1) * ry);

    x2 = (int) (cos(a2) * rx);
    y2 = (int) (-sin(a2) * ry);


    /* Rotate the line: */

    if (rotn != 0)
    {
      rotn_rad = rotn * M_PI / 180;

      xp = x1 * cos(rotn_rad) - y1 * sin(rotn_rad);
      yp = x1 * sin(rotn_rad) + y1 * cos(rotn_rad);

      x1 = xp;
      y1 = yp;

      xp = x2 * cos(rotn_rad) - y2 * sin(rotn_rad);
      yp = x2 * sin(rotn_rad) + y2 * cos(rotn_rad);

      x2 = xp;
      y2 = yp;
    }


    /* Center the line around the center of the shape: */

    x1 = x1 + cx;
    y1 = y1 + cy;
    x2 = x2 + cx;
    y2 = y2 + cy;


    /* Draw: */

    if (!use_brush)
    {
      /* (XOR) */
	
      line_xor(x1, y1, x2, y2);
    }
    else
    {
      /* Brush */

      brush_draw(x1, y1, x2, y2, 0);
    }

#ifdef SCAN_FILL
    fpts_orig[num_pts].x = (float) x2;
    fpts_orig[num_pts].y = (float) y2;
    num_pts++;
#endif
  }


  if (use_brush && shape_filled[cur_shape])
  {
#ifdef SCAN_FILL
    /* FIXME: This is all broken!!! */

    num_pts = clip_polygon(num_pts, fpts_orig, fpts_new);

    for (i = 0; i < num_pts; i++)
    {
      pts[i].x = (int) (fpts_new[i].x);
      pts[i].y = (int) (fpts_new[i].y);
    }

    scan_fill(num_pts, pts);
#else
    /* FIXME: In the meantime, we'll do this lame radius-based fill: */

    for (xx = abs(rx); xx >= 0; xx--)
    {
      for (side = 0; side < shape_sides[cur_shape]; side++)
      {
        a1 = (angle_skip * side + init_ang) * M_PI / 180;
        a2 = (angle_skip * (side + 1) + init_ang) * M_PI / 180;

        x1 = (int) (cos(a1) * xx);
        y1 = (int) (-sin(a1) * ry);

        x2 = (int) (cos(a2) * xx);
        y2 = (int) (-sin(a2) * ry);


        /* Rotate the line: */

        if (rotn != 0)
        {
          rotn_rad = rotn * M_PI / 180;

          xp = x1 * cos(rotn_rad) - y1 * sin(rotn_rad);
          yp = x1 * sin(rotn_rad) + y1 * cos(rotn_rad);

          x1 = xp;
          y1 = yp;

          xp = x2 * cos(rotn_rad) - y2 * sin(rotn_rad);
          yp = x2 * sin(rotn_rad) + y2 * cos(rotn_rad);

          x2 = xp;
          y2 = yp;
        }


        /* Center the line around the center of the shape: */

        x1 = x1 + cx;
        y1 = y1 + cy;
        x2 = x2 + cx;
        y2 = y2 + cy;


        /* Draw: */

        brush_draw(x1, y1, x2, y2, 0);
      }

      if (xx % 10 == 0)
        update_canvas(0, 0, 640 - 96, (48 * 7) + 40);
    }
#endif
  }


  /* Update it! */

  if (use_brush)
  {
    if (abs(rx) > abs(ry))
      rmax = abs(rx) + 20;
    else
      rmax = abs(ry) + 20;

    update_canvas(cx - rmax, cy - rmax,
		  cx + rmax, cy + rmax);
  }


  /* Return to normal brush (for paint brush and line tools): */

  if (use_brush)
  {
    cur_brush = old_brush;
    render_brush();
  }
}


/* What angle is the mouse away from the center of a shape? */

int rotation(int ctr_x, int ctr_y, int ox, int oy)
{
  return(atan2(oy - ctr_y, ox - ctr_x) * 180 / M_PI);
}


/* FIXME: Move elsewhere!!! */

#define PROMPT_SAVE_OVER_TXT gettext("Save over the older version of this drawing?")
#define PROMPT_SAVE_OVER_YES gettext("Yes")
#define PROMPT_SAVE_OVER_NO  gettext("No, save a new file")


/* Save the current image: */

void do_save(void)
{
  int res;
  char * fname;
  char tmp[1024];
#ifndef SAVE_AS_BMP
  png_structp png_ptr;
  png_infop info_ptr;
  unsigned char ** png_rows;
  Uint8 r, g, b;
  int x, y;
  FILE * fi;
#endif


  /* Saving the same picture? */

  if (file_id[0] != '\0')
  {
    /* We sure we want to do that? */
	
    if (do_prompt(PROMPT_SAVE_OVER_TXT,
	          PROMPT_SAVE_OVER_YES,
	          PROMPT_SAVE_OVER_NO) == 0)
    {
      /* No - Let's save a new picture! */

      get_new_file_id();
    }
  }
  else
  {
    /* Saving a new picture: */
	
    get_new_file_id();
  }


  /* Make sure we have a ~/.tuxpaint directory: */

  fname = get_fname("");

  res = mkdir(fname, 0755);

  if (res != 0 && errno != EEXIST)
  {
    fprintf(stderr,
            "\nError: Can't create user data directory:\n"
	    "%s\n"
	    "The error that occured was:\n"
	    "%s\n\n", fname, strerror(errno));

    fprintf(stderr,
            "Cannot save the any pictures! SORRY!\n\n");

    draw_tux_text(TUX_OOPS, SDL_GetError());

    free(fname);
    return;
  }
  free(fname);


  /* Make sure we have a ~/.tuxpaint/saved directory: */

  fname = get_fname("saved");

  res = mkdir(fname, 0755);

  if (res != 0 && errno != EEXIST)
  {
    fprintf(stderr,
            "\nError: Can't create user data directory:\n"
	    "%s\n"
	    "The error that occured was:\n"
	    "%s\n\n", fname, strerror(errno));

    fprintf(stderr,
            "Cannot save the any pictures! SORRY!\n\n");

    draw_tux_text(TUX_OOPS, SDL_GetError());

    free(fname);
    return;
  }
  free(fname);


  /* Save the file: */

  snprintf(tmp, sizeof(tmp), "saved/%s%s", file_id, FNAME_EXTENSION);
  fname = get_fname(tmp);

#ifdef SAVE_AS_BMP
  if (SDL_SaveBMP(canvas, fname))
  {
    fprintf(stderr,
            "\nError: Couldn't save the current image!\n"
	    "%s\n"
	    "The Simple DirectMedia Layer error that occured was:\n"
	    "%s\n\n", fname, SDL_GetError());

    draw_tux_text(TUX_OOPS, SDL_GetError());
  }
  else
  {
    /* Ta-Da! */

    playsound(0, SND_SAVE, 1);
    draw_tux_text(TUX_DEFAULT, gettext(tool_tips[TOOL_SAVE]));
  }
#else
  fi = fopen(fname, "w");
  if (fi == NULL)
  {
    fprintf(stderr,
	    "\nError: Couldn't save the current image!\n"
	    "%s\n"
	    "The system error that occured was:\n"
	    "%s\n\n",
	    fname, strerror(errno));

    draw_tux_text(TUX_OOPS, strerror(errno));
  }
  else
  {
    png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
    if (png_ptr == NULL)
    {
      fclose(fi);
      png_destroy_write_struct(&png_ptr, (png_infopp) NULL);

      fprintf(stderr, "\nError: Couldn't save the image!\n%s\n\n", fname);
      draw_tux_text(TUX_OOPS, strerror(errno));
    }
    else
    {
      info_ptr = png_create_info_struct(png_ptr);
      if (info_ptr == NULL)
      {
        fclose(fi);
        png_destroy_write_struct(&png_ptr, (png_infopp) NULL);

        fprintf(stderr, "\nError: Couldn't save the image!\n%s\n\n", fname);
        draw_tux_text(TUX_OOPS, strerror(errno));
      }
      else
      {
	if (setjmp(png_jmpbuf(png_ptr)))
	{
	  fclose(fi);
	  png_destroy_write_struct(&png_ptr, (png_infopp) NULL);

	  fprintf(stderr, "\nError: Couldn't save the image!\n%s\n\n", fname);
	  draw_tux_text(TUX_OOPS, strerror(errno));
	}
	else
	{
	  png_init_io(png_ptr, fi);

	  info_ptr->width = canvas->w;
	  info_ptr->height = canvas->h;
	  info_ptr->bit_depth = 8;
	  info_ptr->color_type = PNG_COLOR_TYPE_RGB;
	  info_ptr->interlace_type = 1;
	  info_ptr->valid = 0;

	  png_write_info(png_ptr, info_ptr);


	  /* Save the picture: */

	  png_rows = malloc(sizeof(char *) * canvas->h);

	  for (y = 0; y < canvas->h; y++)
	  {
	    png_rows[y] = malloc(sizeof(char) * 3 * canvas->w);

	    for (x = 0; x < canvas->w; x++)
	    {
	      SDL_GetRGB(getpixel(canvas, x, y), canvas->format, &r, &g, &b);

	      png_rows[y][x * 3 + 0] = r;
	      png_rows[y][x * 3 + 1] = g;
	      png_rows[y][x * 3 + 2] = b;
	    }
	  }
	  
	  png_write_image(png_ptr, png_rows);

	  for (y = 0; y < canvas->h; y++)
	    free(png_rows[y]);

	  free(png_rows);
	  

	  png_write_end(png_ptr, NULL);

	  png_destroy_write_struct(&png_ptr, &info_ptr);
	  fclose(fi);
    
	  /* Ta-Da! */

	  playsound(0, SND_SAVE, 1);
	  draw_tux_text(TUX_DEFAULT, gettext(tool_tips[TOOL_SAVE]));
	}
      }
    }
  }
#endif
  free(fname);


}


/* Pick a new file ID: */

void get_new_file_id(void)
{
  time_t t;

  t = time(NULL);

  strftime(file_id, sizeof(file_id), "%Y%m%d%H%M%S", localtime(&t));
  debug(file_id);


  /* FIXME: Show thumbnail and prompt for title: */
}


/* Handle quitting (and prompting to save, if necessary!) */

int do_quit(void)
{
  int done;

  done = do_prompt(PROMPT_QUIT_TXT,
	           PROMPT_QUIT_YES,
		   PROMPT_QUIT_NO);

  if (done && !been_saved)
  {
    if (do_prompt(PROMPT_QUIT_SAVE_TXT,
	          PROMPT_QUIT_SAVE_YES,
	          PROMPT_QUIT_SAVE_NO))
    {
      do_save();
    }
  }

  return(done);
}


/* Open a saved image: */

void do_open(void)
{
  SDL_Surface * img, * img1, * img2;
  SDL_Surface * thumbs[MAX_FILES];
  DIR * d;
  struct dirent * f;
  char * dirname, * rfname;
  char * d_names[MAX_FILES], * d_exts[MAX_FILES];
  char fname[1024];
  int num_files, i, done, update_list, want_erase, cur, which;
  int thumb_w, thumb_h;
  SDL_Rect dest;
  SDL_Event event;
  SDLKey key;


  thumb_w = (640 - 96 - 96) / 4;
  thumb_h = ((48 * 7 + 40) - 72) / 4;


  /* Open directory of images: */

  dirname = get_fname("saved");

  d = opendir(dirname);
  if (d == NULL)
  {
    fprintf(stderr,
	    "\nWarning: There's no directory of saved images\n"
	    "%s\n"
	    "The system error that occured was: %s\n",
	    dirname, strerror(errno));
  }


  /* Read directory of images and build thumbnails: */

  num_files = 0;

  if (d != NULL)
  {
    dest.x = (640 - img_loading->w) / 2;
    dest.y = (480 - img_loading->h) / 2;
    SDL_BlitSurface(img_loading, NULL, screen, &dest);

    SDL_UpdateRect(screen, dest.x, dest.y, img_loading->w, img_loading->h);

    do
    {
      f = readdir(d);

      show_progress_bar();

      if (f != NULL)
      {
        debug(f->d_name);
        if (strstr(f->d_name, FNAME_EXTENSION) != NULL
#ifndef SAVE_AS_BMP
	    /* Support legacy BMP files for load: */
	
	    || strstr(f->d_name, ".bmp") != NULL
#endif
            )
        {
          strcpy(fname, f->d_name);
	  if (strstr(fname, FNAME_EXTENSION) != NULL)
	    {
	      strcpy(strstr(fname, FNAME_EXTENSION), "");
	      d_exts[num_files] = strdup(FNAME_EXTENSION);
	    }

#ifndef SAVE_AS_BMP
	  if (strstr(fname, ".bmp") != NULL)
	    {
	      strcpy(strstr(fname, ".bmp"), "");
	      d_exts[num_files] = strdup(".bmp");
	    }
#endif
	
          d_names[num_files] = strdup(fname);

          snprintf(fname, sizeof(fname), "%s/%s", dirname, f->d_name);
          debug(fname);
#ifdef SAVE_AS_BMP
          img = SDL_LoadBMP(fname);
#else
          img = IMG_Load(fname);
#endif
      
	  show_progress_bar();

          if (img == NULL)
          {
	    fprintf(stderr,
		    "\nWarning: I can't open one of the saved files!\n"
		    "%s\n"
	 	    "The Simple DirectMedia Layer error that occured was:\n"
		    "%s\n\n",
		    fname, SDL_GetError());

	    free(d_names[num_files]);
	    free(d_exts[num_files]);
          }
          else
          {
	    /* Turn it into a thumbnail: */

	    img1 = SDL_DisplayFormat(img);
  	    img2 = thumbnail(img1, thumb_w - 20, thumb_h - 20, 0);
	    SDL_FreeSurface(img1);
      
	    show_progress_bar();
	
            thumbs[num_files] = SDL_DisplayFormat(img2);
	    SDL_FreeSurface(img2);
	    if (thumbs[num_files] == NULL)
	    {
	      fprintf(stderr,
		      "\nError: Couldn't create a thumbnail of saved image!\n"
		      "%s\n", fname);
	    }

	    SDL_FreeSurface(img);
      
	    show_progress_bar();

	    num_files++;
	  }
        }
      }
    }
    while (f != NULL && num_files < MAX_FILES);

    closedir(d);
  }

  free(dirname);


#ifdef DEBUG
  printf("%d saved files were found!\n", num_files);
#endif

  if (num_files == 0)
  {
    do_prompt(PROMPT_OPEN_NOFILES_TXT, PROMPT_OPEN_NOFILES_YES, "");
  }
  else
  {
    /* Let user choose an image: */

    draw_tux_text(TUX_BORED,
		  gettext("Choose the picture you want, then click 'Open'"));

    cur = 0;
    update_list = 1;
    want_erase = 0;

    done = 0;
    which = 0;

    do
    {
      /* Update screen: */
	
      if (update_list)
      {
	/* Erase: */
	
        dest.x = 96;
        dest.y = 0;
        dest.w = 640 - 96 - 96;
        dest.h = 48 * 7 + 40;

        SDL_FillRect(screen, &dest, SDL_MapRGB(screen->format, 255, 255, 255));


	/* Draw icons: */
	
        for (i = cur; i < cur + 16 && i < num_files; i++)
        {
	  /* Draw cursor: */

          dest.x = thumb_w * ((i - cur) % 4) + 96;
          dest.y = thumb_h * ((i - cur) / 4) + 24;

	  if (i == which)
	    SDL_BlitSurface(img_cursor_down, NULL, screen, &dest);
	  else
	    SDL_BlitSurface(img_cursor_up, NULL, screen, &dest);

	  
          debug(d_names[i]);

          dest.x = thumb_w * ((i - cur) % 4) + 96 + 10;
          dest.y = thumb_h * ((i - cur) / 4) + 24 + 10;
	  
          if (thumbs[i] != NULL)
            SDL_BlitSurface(thumbs[i], NULL, screen, &dest);
        }


	/* Draw arrows: */

	dest.x = (640 - img_scroll_up->w) / 2;
	dest.y = 0;
	
	if (cur > 0)
	  SDL_BlitSurface(img_scroll_up, NULL, screen, &dest);
	else
	  SDL_BlitSurface(img_scroll_up_off, NULL, screen, &dest);

	dest.x = (640 - img_scroll_up->w) / 2;
	dest.y = (48 * 7 + 40) - 48;
	
	if (cur < num_files - 16)
	  SDL_BlitSurface(img_scroll_down, NULL, screen, &dest);
	else
	  SDL_BlitSurface(img_scroll_down_off, NULL, screen, &dest);

	dest.x = 96;
	dest.y = (48 * 7 + 40) - 48;
	SDL_BlitSurface(img_open, NULL, screen, &dest);
	
	dest.x = 640 - 96 - 48;
	dest.y = (48 * 7 + 40) - 48;
	SDL_BlitSurface(img_back, NULL, screen, &dest);
	
	dest.x = 640 - 96 - 48 - 48;
	dest.y = (48 * 7 + 40) - 48;
	SDL_BlitSurface(img_erase, NULL, screen, &dest);
	
        SDL_Flip(screen);

	update_list = 0;
      }


      SDL_WaitEvent(&event);

      if (event.type == SDL_QUIT)
      {
	done = 1;

	/* FIXME: Handle SDL_Quit better */
      }
      else if (event.type == SDL_KEYDOWN)
      {
	key = event.key.keysym.sym;

	if (key == SDLK_LEFT)
	{
	  if (which > 0)
	  {
	    which--;

	    if (which < cur)
	      cur = cur - 4;

	    update_list = 1;
	  }
	}
	else if (key == SDLK_RIGHT)
	{
	  if (which < num_files - 1)
	  {
	    which++;

	    if (which >= cur + 16)
	      cur = cur + 4;
	
	    update_list = 1;
	  }
	}
	else if (key == SDLK_UP)
	{
	  if (which >= 0)
	  {
	    which = which - 4;
	
	    if (which < 0)
	      which = 0;

	    if (which < cur)
	      cur = cur - 4;

	    update_list = 1;
	  }
	}
	else if (key == SDLK_DOWN)
	{
	  if (which < num_files)
	  {
	    which = which + 4;

	    if (which >= num_files)
	      which = num_files - 1;

	    if (which >= cur + 16)
	      cur = cur + 4;

	    update_list = 1;
	  }
	}
	else if (key == SDLK_RETURN || key == SDLK_SPACE)
	{
	  /* Open */

	  done = 1;
	  playsound(1, SND_CLICK, 1);
	}
	else if (key == SDLK_ESCAPE)
	{
	  /* Go back: */

	  which = -1;
	  done = 1;
	  playsound(1, SND_CLICK, 1);
	}
	else if (key == SDLK_d &&
		 (event.key.keysym.mod == KMOD_CTRL ||
		  event.key.keysym.mod == KMOD_LCTRL ||
		  event.key.keysym.mod == KMOD_RCTRL))
	{
	  /* Delete! */

	  want_erase = 1;
	}
      }
      else if (event.type == SDL_MOUSEBUTTONDOWN)
      {
	if (event.button.x >= 96 && event.button.x < 640 - 96 &&
	    event.button.y >= 24 && event.button.y < (48 * 7 + 40 - 48))
	{
	  /* Picked an icon! */

          which = ((event.button.x - 96) / (thumb_w) +
		   (((event.button.y - 24) / thumb_h) * 4)) + cur;

	  if (which < num_files)
	  {
	    playsound(1, SND_BLEEP, 1);
	    update_list = 1;
	  }
	}
	else if (event.button.x >= (640 - img_scroll_up->w) / 2 &&
	         event.button.x <= (640 + img_scroll_up->w) / 2)
	{
	  if (event.button.y < 24)
	  {
	    /* Up scroll button: */
		
	    if (cur > 0)
	    {
              cur = cur - 4;
	      update_list = 1;
	      playsound(1, SND_SCROLL, 1);
	    }

	    if (which > cur + 16)
	      which = which - 4;
	  }
	  else if (event.button.y >= (48 * 7 + 40 - 48) &&
		   event.button.y < (48 * 7 + 40 - 24))
	  {
	    /* Down scroll button: */
		
	    if (cur < num_files - 16)
	    {
	      cur = cur + 4;
	      update_list = 1;
	      playsound(1, SND_SCROLL, 1);
	    }
	
	    if (which < cur)
	      which = which + 4;
	  }
	}
	else if (event.button.x >= 96 && event.button.x < 96 + 48 &&
		 event.button.y >= (48 * 7 + 40) - 48 &&
		 event.button.y < (48 * 7 + 40))
	{
	  /* Open */

	  done = 1;
	  playsound(1, SND_CLICK, 1);
	}
	else if (event.button.x >= (640 - 96 - 48) &&
		 event.button.x < (640 - 96) &&
		 event.button.y >= (48 * 7 + 40) - 48 &&
		 event.button.y < (48 * 7 + 40))
	{
	  /* Back */

	  which = -1;
	  done = 1;
	  playsound(1, SND_CLICK, 1);
	}
	else if (event.button.x >= (640 - 96 - 48 - 48) &&
		 event.button.x < (640 - 48 - 96) &&
		 event.button.y >= (48 * 7 + 40) - 48 &&
		 event.button.y < (48 * 7 + 40))
	{
	  /* Erase */

	  want_erase = 1;
	}
      }

      if (want_erase)
      {
	want_erase = 0;

        if (do_prompt(PROMPT_ERASE_TXT, PROMPT_ERASE_YES, PROMPT_ERASE_NO))
        {
          /* FIXME: Erase the file! */

          snprintf(fname, sizeof(fname), "saved/%s%s",
		   d_names[which], d_exts[which]);

          rfname = get_fname(fname);
          debug(rfname);

          if (unlink(rfname) == 0)
          {
            thumbs[which] = NULL;
            update_list = 1;


            /* Move all other files up a notch: */

            free(d_names[which]);
            free(d_exts[which]);
            free_surface(&thumbs[which]);

            for (i = which; i < num_files - 1; i++)
            {
              d_names[i] = d_names[i + 1];
              d_exts[i] = d_exts[i + 1];
              thumbs[i] = thumbs[i + 1];
            }

            num_files--;


            /* Make sure the cursor doesn't go off the end! */

            if (which >= num_files)
              which = num_files - 1;


            /* No files to open now? */

            if (which < 0)
            {
              do_prompt(PROMPT_OPEN_NOFILES_TXT, PROMPT_OPEN_NOFILES_YES, "");
              done = 1;
            }
          }
          else
          {
            perror(rfname);

            do_prompt("CAN'T", "OK", "");
	    update_list = 1;
          }

          free(rfname);
        }
        else
        {
          update_list = 1;
        }
      }

    }
    while (!done);


    /* Load the chosen picture: */

    if (which != -1)
    {
      /* Save old one first? */
	
      if (!been_saved)
      {
        if (do_prompt(PROMPT_OPEN_SAVE_TXT,
                      PROMPT_OPEN_SAVE_YES,
                      PROMPT_OPEN_SAVE_NO))
        {
          do_save();
        }
      }

      snprintf(fname, sizeof(fname), "saved/%s%s",
	       d_names[which], d_exts[which]);

      rfname = get_fname(fname);

#ifdef SAVE_AS_BMP
      img = SDL_LoadBMP(rfname);
#else
      img = IMG_Load(rfname);
#endif

      if (img == NULL)
      {
	fprintf(stderr,
		"\nWarning: Couldn't load the saved image!\n"
		"%s\n"
		"The Simple DirectMedia Layer error that occured was:\n"
		"%s\n\n", fname, SDL_GetError());

	do_prompt(PROMPT_OPEN_UNOPENABLE_TXT,
		  PROMPT_OPEN_UNOPENABLE_YES, "");
      }
      else
      {
	SDL_BlitSurface(img, NULL, canvas, NULL);
	SDL_FreeSurface(img);
	
	cur_undo = 0;
	oldest_undo = 0;
	newest_undo = 0;
	
	been_saved = 1;
	reset_avail_tools();
	tool_avail[TOOL_NEW] = 1;
	
	strcpy(file_id, d_names[which]);
      }

      free(rfname);
    }


    update_canvas(0, 0, 640 - 96 - 96, 48 * 7 + 40);
  }


  /* Clean up: */

  free_surface_array(thumbs, num_files);

  for (i = 0; i < num_files; i++)
  {
    free(d_names[i]);
    free(d_exts[i]);
  }
}


/* -------------- Poly Fill Stuff -------------- */

void insert_edge(edge * list, edge * edg)
{
  edge * p, * q;

  debug("insert_edge()");

  q = list;

  p = q->next;
  while (p != NULL)
  {
    if (edg->x_intersect < p->x_intersect)
    {
      p = NULL;
    }
    else
    {
      q = p;
      p = p->next;
    }
  }

  edg->next = q->next;
  q->next = edg;
}


int y_next(int k, int cnt, point_type * pts)
{
  int j;

  debug("y_next()");

  if ((k + 1) > (cnt - 1))
    j = 0;
  else
    j = k + 1;

  while (pts[k].y == pts[j].y)
  {
    if ((j + 1) > (cnt - 1))
      j = 0;
    else
      j++;
  }

  return (pts[j].y);
}


void make_edge_rec(point_type lower, point_type upper,
		   int y_comp, edge * edg, edge * edges[])
{
  debug("make_edge_rec()");

  edg->dx_per_scan = (float)((upper.x - lower.x) / (upper.y - lower.y));
  edg->x_intersect = lower.x;

  if (upper.y < y_comp)
    edg->y_upper = upper.y - 1;
  else
    edg->y_upper = upper.y;

  insert_edge(edges[lower.y], edg);
}


void build_edge_list(int cnt, point_type * pts, edge * edges[])
{
  edge * edg;
  point_type v1, v2;
  int i, y_prev;

  debug("build_edge_list()");

  y_prev = pts[cnt - 2].y;

  v1.x = pts[cnt - 1].x;
  v1.y = pts[cnt - 1].y;

  for (i = 0; i < cnt; i++)
  {
    v2 = pts[i];
    if (v1.y != v2.y)
    {
      edg = (edge *) malloc(sizeof(edge));
      if (v1.y < v2.y)
	make_edge_rec(v1, v2, y_next(i, cnt, pts), edg, edges);
      else
	make_edge_rec(v2, v1, y_prev, edg, edges);
    }

    y_prev = v1.y;
    v1 = v2;
  }
}


void build_active_list(int scan, edge * active, edge * edges[])
{
  edge * p, * q;

  debug("build_active_list()");

  p = edges[scan]->next;
  while (p != NULL)
  {
    q = p->next;
    insert_edge(active, p);
    p = q;
  }
}


void fill_scan(int scan, edge * active)
{
  edge * p1, * p2;
  int i;
  Uint32 color;


  debug("fill_scan()");

  color = SDL_MapRGB(canvas->format,
		     color_hexes[cur_color][0] / 2,
		     color_hexes[cur_color][1] / 2,
		     color_hexes[cur_color][2] / 2);

  SDL_LockSurface(canvas);

  p1 = active->next;
  while (p1 != NULL)
  {
    p2 = p1->next;

    for (i = p1->x_intersect; i < p2->x_intersect; i++)
    {
      putpixel(canvas, i, scan, color);
    }

    p1 = p2->next;
  }

  SDL_UnlockSurface(canvas);
}


void delete_after(edge * q)
{
  edge * p;

  debug("delete_after()");

  p = q->next;
  q->next = p->next;
  free(p);
}


void update_active_list(int scan, edge * active)
{
  edge * q, * p;

  debug("update_active_list()");

  q = active;
  p = active->next;

  while (p != NULL)
  {
    if (scan >= p->y_upper)
    {
      p = p->next;
      delete_after(q);
    }
    else
    {
      p->x_intersect = p->x_intersect + p->dx_per_scan;
      q = p;
      p = p->next;
    }
  }
}


void resort_active_list(edge * active)
{
  edge * q, * p;

  debug("resort_active_list()");

  p = active->next;

  active->next = NULL;

  while (p != NULL)
  {
    q = p->next;
    insert_edge(active, p);
    p = q;
  }
}


void scan_fill(int cnt, point_type * pts)
{
  edge * edges[48 * 7 + 40 + 5], * active;
  int i, scan;

  debug("scan_fill()");


  /* Create empty edges: */

  for (i = 0; i < 48 * 7 + 40 + 5; i++)
  {
    edges[i] = (edge *) malloc(sizeof(edge));
    edges[i]->next = NULL;
  }


  /* Build edge list: */

  build_edge_list(cnt, pts, edges);


  /* Set active edge: */

  active = (edge *) malloc(sizeof(edge));
  active->next = NULL;


  /* Scan! */

  for (scan = 0; scan < 48 * 7 + 40; scan++)
  {
    build_active_list(scan, active, edges);

    if (active->next)
    {
      fill_scan(scan, active);

      update_canvas(0, scan, 640 - 96, scan);
      SDL_Flip(screen);
      SDL_Delay(10);

      update_active_list(scan, active);
      resort_active_list(active);
    }
  }


  /* Free edge list: */

  debug("Freeing...");

  for (i = 0; i < 48 * 7 + 40; i++)
  {
    free(edges[i]);
  }
}


/* ------------- Poly clipping stuff: -------------- */

int inside(fpoint_type p, an_edge b)
{
  if (b == Left)
  {
    if (p.x < 0)
      return 0;
  }
  else if (b == Right)
  {
    if (p.x >= 640 - 96)
      return 0;
  }
  else if (b == Bottom)
  {
    if (p.y >= 48 * 7 + 40)
      return 0;
  }
  else if (b == Top)
  {
    if (p.y < 0)
      return 0;
  }

  return 1;
}


int cross(fpoint_type p1, fpoint_type p2, an_edge b)
{
  if (inside(p1, b) == inside(p2, b))
    return 0;
  else
    return 1;
}


fpoint_type intersect(fpoint_type p1, fpoint_type p2, an_edge b)
{
  fpoint_type ipt;
  float m;

  if (p1.x != p2.x)
    m = (p1.y - p2.y) / (p1.x - p2.x);
  else
    m = 1.0;

  if (b == Left)
  {
    ipt.x = 0;
    ipt.y = p2.y + (-p2.x) * m;
  }
  else if (b == Right)
  {
    ipt.x = 640 - 96 - 1;
    ipt.y = p2.y + ((640 - 96 - 1) - p2.x) * m;
  }
  else if (b == Top)
  {
    ipt.y = 0;
    if (p1.x != p2.x)
      ipt.x = p2.x + (-p2.y) / m;
    else
      ipt.x = p2.x;
  }
  else if (b == Bottom)
  {
    ipt.y = (48 * 7 + 40) - 1;
    if (p1.x != p2.x)
      ipt.x = p2.x + (((48 * 7 + 40) - 1) - p2.y) / m;
    else
      ipt.x = p2.x;
  }

  return(ipt);
}


void clip_point(fpoint_type p, an_edge b, fpoint_type * pout, int * cnt,
		fpoint_type * first[], fpoint_type * s)
{
  fpoint_type ipt;

  if (first[b] == NULL)
  {
    first[b] = &p;
  }
  else
  {
    if (cross(p, s[b], b))
    {
      ipt = intersect(p, s[b], b);

      if (b < Top)  /* Should be NUM_EDGES? */
      {
        clip_point(ipt, b + 1, pout, cnt, first, s);
      }
      else
      {
	pout[*cnt] = ipt;
	(*cnt)++;
      }
    }
  }

  s[b] = p;


  if (inside(p, b))
  {
    if (b < Top)  /* Should be NUM_EDGES? */
    {
      clip_point(p, b + 1, pout, cnt, first, s);
    }
    else
    {
      pout[*cnt] = p;
      (*cnt)++;
    }
  }
}


void close_clip(fpoint_type * pout, int * cnt, fpoint_type * first[],
		fpoint_type * s)
{
  fpoint_type i;
  an_edge b;

  for (b = Left; b <= Top; b++)
  {
    if (cross(s[b], *first[b], b))
    {
      i = intersect(s[b], *first[b], b);
      if (b < Top)
      {
	clip_point(i, b + 1, pout, cnt, first, s);
      }
      else
      {
	pout[*cnt] = i;
	(*cnt)++;
      }
    }
  }
}


int clip_polygon(int n, fpoint_type * pin, fpoint_type * pout)
{
  fpoint_type * first[NUM_EDGES] = {0, 0, 0, 0};
  fpoint_type s[NUM_EDGES];
  int i, cnt;

  cnt = 0;

  for (i = 0; i < n; i++)
  {
    clip_point(pin[i], Left, pout, &cnt, first, s);
  }

  close_clip(pout, &cnt, first, s);

  return(cnt);
}


/* Let sound effects (e.g., "Save" sfx) play out before quitting... */

void wait_for_sfx(void)
{
#ifndef NOSOUND
  if (use_sound)
  {
    while (Mix_Playing(-1))
      SDL_Delay(10);
  }
#endif
}


/* Determine the current language/locale, and set the language string: */

int current_language(void)
{
  char * loc;
  int lang, i;


  /* Default... */

  lang = LANG_EN;


  loc = setlocale(LC_ALL, NULL);
  if (loc != NULL)
  {
    if (strstr(loc, "LC_MESSAGES") != NULL)
      loc = getenv("LANG");
  }

  if (loc != NULL)
  {
    /* Which, if any, of the locales is it? */

    for (i = 1; i < NUM_LANGS; i++)
    {
      if (strstr(loc, lang_prefixes[i]) == loc)
      {
	lang = i;
      }
    }
  }


  return lang;
}


/* XOR-based outline of rubber stamp shapes
   (unused if LOW_QUALITY_STAMP_OUTLINE is #defined) */

void stamp_xor(int x, int y)
{
  int xx, yy;
  Uint8 r, g, b, a, olda, abovea;

  SDL_LockSurface(img_stamps[cur_stamp]);
  SDL_LockSurface(screen);

  for (yy = (y % 2) + 1; yy < img_stamps[cur_stamp]->h; yy = yy + 2)
  {
    olda = 0;

    for (xx = (x % 2); xx < img_stamps[cur_stamp]->w; xx = xx + 2)
    {
      SDL_GetRGBA(getpixel(img_stamps[cur_stamp], xx, yy),
		  img_stamps[cur_stamp]->format, &r, &g, &b, &a);
      
      SDL_GetRGBA(getpixel(img_stamps[cur_stamp], xx, yy - 1),
		  img_stamps[cur_stamp]->format, &r, &g, &b, &abovea);

      if ((a < 128 && olda >= 128) ||
          (a >= 128 && olda < 128) ||
	  (a < 128 && abovea >= 128) ||
	  (a >= 128 && abovea < 128))
      {
        clipped_putpixel(screen, x + 96 + xx, y + yy,
			 0xFFFFFFFF - getpixel(screen, x + 96 + xx, y + yy));
      }

      olda = a;
    }
  }

  SDL_UnlockSurface(screen);
  SDL_UnlockSurface(img_stamps[cur_stamp]);
}


/* Returns whether a particular stamp can be colored: */

int stamp_colorable(int stamp)
{
  if (inf_stamps[stamp] != NULL)
  {
    return inf_stamps[stamp]->colorable;
  }
  else
  {
    return 0;
  }
}


/* Returns whether a particular stamp can be tinted: */

int stamp_tintable(int stamp)
{
  if (inf_stamps[stamp] != NULL)
  {
    return inf_stamps[stamp]->tintable;
  }
  else
  {
    return 0;
  }
}



void rgbtohsv(Uint8 r8, Uint8 g8, Uint8 b8, float *h, float *s, float *v)
{
  float rgb_min, rgb_max, delta, r, g, b;
  
  r = (r8 / 255.0);
  g = (g8 / 255.0);
  b = (b8 / 255.0);
  
  rgb_min = min(r, min(g, b));
  rgb_max = max(r, max(g, b));
  *v = rgb_max;
  
  delta = rgb_max - rgb_min;

  if (rgb_max == 0)
    {
      /* Black */
      
      *s = 0;
      *h = -1;
    }
  else
    {
      *s = delta / rgb_max;
      
      if (r == rgb_max)
	*h = (g - b) / delta;
      else if (g == rgb_max)
	*h = 2 + (b - r) / delta;     /* between cyan & yellow */
      else
	*h = 4 + (r - g) / delta;     /* between magenta & cyan */
      
      *h = (*h * 60);                 /* degrees */
      
      if (*h < 0)
	*h = (*h + 360);
    }
}


void hsvtorgb(float h, float s, float v, Uint8 *r8, Uint8 *g8, Uint8 *b8)
{
  int i;
  float f, p, q, t, r, g, b;
  
  if (s == 0)
    {
      /* Achromatic (grey) */
      
      r = v;
      g = v;
      b = v;
    }
  else
    {
      h = h / 60;
      i = floor(h);
      f = h - i;
      p = v * (1 - s);
      q = v * (1 - s * f);
      t = v * (1 - s * (1 - f));
      
      if (i == 0)
	{
	  r = v;
	  g = t;
	  b = p;
	}
      else if (i == 1)
	{
	  r = q;
	  g = v;
	  b = p;
	}
      else if (i == 2)
	{
	  r = p;
	  g = v;
	  b = t;
	}
      else if (i == 3)
	{
	  r = p;
	  g = q;
	  b = v;
	}
      else if (i == 4)
	{
	  r = t;
	  g = p;
	  b = v;
	}
      else
	{
	  r = v;
	  g = p;
	  b = q;
	}
    }

  
  *r8 = (Uint8) (r * 255);
  *g8 = (Uint8) (g * 255);
  *b8 = (Uint8) (b * 255);
}


void show_progress_bar(void)
{
  SDL_Rect dest, src;
  int x;


  for (x = 0; x < 640; x = x + 65)
  {
    src.x = (prog_bar_ctr % 65);
    src.y = 0;
    src.w = 65;
    src.h = 24;

    dest.x = x;
    dest.y = 480 - 24;
  
    SDL_BlitSurface(img_progress, &src, screen, &dest);
  }

  prog_bar_ctr++;

  SDL_UpdateRect(screen, 0, 480 - 24, 640, 24);
}
