/* 
 * Copyright (C) 2000-2002 the xine project
 * 
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: lirc.c,v 1.14 2002/06/02 21:59:55 f1rmb Exp $
 *
 * lirc-specific stuff
 *
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <pthread.h>
#include <signal.h>

#include "event.h"
#include "panel.h"
#include "actions.h"
#include "playlist.h"

#include "xitk.h"

#ifdef HAVE_LIRC
#include "lirc/lirc_client.h"

#ifdef DEBUG
#define LIRC_VERBOSE 1
#else
#define LIRC_VERBOSE 0
#endif

extern gGui_t          *gGui;

extern _panel_t        *panel;

typedef struct {
  struct lirc_config   *config;
  int                   fd;
  pthread_t             thread;
} _lirc_t;

static _lirc_t lirc;

static void lirc_get_playlist(char *from) {
  int    i;
  char **autoscan_plugins = xine_get_autoplay_input_plugin_ids(gGui->xine);

  for(i = 0; autoscan_plugins[i] != NULL; ++i) {
    if(!strcasecmp(autoscan_plugins[i], from)) {
      int    num_mrls;
      int    j;
      char **autoplay_mrls = xine_get_autoplay_mrls (gGui->xine,
						     from,
						     &num_mrls);
      if(autoplay_mrls) {

	/* First, free existing playlist */
	for(j = 0; j < gGui->playlist_num; j++)
	  gGui->playlist[j] = NULL;
	
	gGui->playlist_num = 0;
	gGui->playlist_cur = 0;
	
	for (j = 0; j < num_mrls; j++)
	  gGui->playlist[j] = autoplay_mrls[j];
	
	gGui->playlist_num = j;
	gGui->playlist_cur = 0;
	gui_set_current_mrl(gGui->playlist[gGui->playlist_cur]);
	pl_update_playlist();
      }    
    }
  }
}

void *xine_lirc_loop(void *dummy) {
  char             *code, *c;
  int               ret;
  kbinding_entry_t *k;

  /* I'm a poor lonesome pthread... */
  pthread_detach(pthread_self());

  while(gGui->running) {
    
    pthread_testcancel();
    
    while(lirc_nextcode(&code) == 0) {

      pthread_testcancel();
      
      if(code == NULL) 
	break;
      
      pthread_testcancel();
      
      while((ret = lirc_code2char(lirc.config, code, &c)) == 0 
	    && c != NULL) {
#if 0
	fprintf(stdout, "Command Received = '%s'\n", c);
#endif

	k = kbindings_lookup_action(gGui->kbindings, c);
	if(k)
	  gui_execute_action_id((kbindings_get_action_id(k)));
	else {
	  char from[256];
	  
	  memset(&from, 0, sizeof(from));
	  if(sscanf(c, "PlaylistFrom:%s", &from[0]) == 1) {
	    if(strlen(from))
	      lirc_get_playlist(from);   
	  }
	}
	
      }
      xitk_paint_widget_list (panel->widget_list);
      free(code);

      if(ret == -1) 
	break;

    }
  }
  
  pthread_exit(NULL);
}

void init_lirc(void) {
  /*  int flags; */

  if((lirc.fd = lirc_init("xine", LIRC_VERBOSE)) == -1) {
    gGui->lirc_enable = 0;
    return;
  }
  /*
  else {
    flags = fcntl(gGui->lirc.fd, F_GETFL, 0);
    if(flags != -1)
      fcntl(gGui->lirc.fd, F_SETFL, flags|FASYNC|O_NONBLOCK);
  }
  */

  if(lirc_readconfig(NULL, &lirc.config, NULL) != 0) {
    gGui->lirc_enable = 0;
    return;
  }

  gGui->lirc_enable = 1;

  if(gGui->lirc_enable)
    pthread_create (&lirc.thread, NULL, xine_lirc_loop, NULL) ;
}

void deinit_lirc(void) {

  pthread_cancel(lirc.thread);

  if(gGui->lirc_enable) {
    lirc_freeconfig(lirc.config);
    lirc_deinit();
  }
}

#endif
